"""Synthetic data testing.
"""

import copy
import unittest
from .network_objects import network_lattice_1x1_no_args
from .network_objects import network_lattice_2x1x1_all
from .network_objects import network_lattice_2x1x1_largest
from .network_objects import network_lattice_1x1_geomelem
from .network_objects import graph_barb, network_barb


class TestNetworkBuildLattice1x1(unittest.TestCase):
    def setUp(self):
        self.network = network_lattice_1x1_no_args

    def test_lattice_network_sdata(self):
        known_segments = 4
        observed_segments = self.network.s_data.shape[0]
        self.assertEqual(observed_segments, known_segments)

        known_length = 18.0
        observed_length = self.network.s_data.length.sum()
        self.assertEqual(observed_length, known_length)

    def test_lattice_network_ndata(self):
        known_nodes = 5
        observed_nodes = self.network.n_data.shape[0]
        self.assertEqual(observed_nodes, known_nodes)

        known_bounds = [0.0, 0.0, 9.0, 9.0]
        observed_bounds = list(self.network.n_data.total_bounds)
        self.assertEqual(observed_bounds, known_bounds)

    def test_lattice_network_sdata_ids(self):
        known_ids = [0, 1, 2, 3]
        observed_ids = list(self.network.s_data["SegID"])
        self.assertEqual(observed_ids, known_ids)

    def test_lattice_network_ndata_ids(self):
        known_ids = [0, 1, 2, 3, 4]
        observed_ids = list(self.network.n_data["NodeID"])
        self.assertEqual(observed_ids, known_ids)

    def test_lattice_network_segm2xyid(self):
        known_xyid = {3: ["x4.5y4.5", "x9.0y4.5"]}
        observed_xyid = self.network.segm2xyid[3]
        self.assertEqual(observed_xyid, known_xyid[3])

    def test_lattice_network_node2xyid(self):
        known_xyid = {4: ["x9.0y4.5"]}
        observed_xyid = self.network.node2xyid[4]
        self.assertEqual(observed_xyid, known_xyid[4])

    def test_lattice_network_s_ids(self):
        known_ids = [0, 1, 2, 3]
        observed_ids = self.network.s_ids
        self.assertEqual(observed_ids, known_ids)

    def test_lattice_network_n_ids(self):
        known_ids = [0, 1, 2, 3, 4]
        observed_ids = self.network.n_ids
        self.assertEqual(observed_ids, known_ids)

    def test_lattice_network_n_segm(self):
        known_segm_count, observed_segm_count = 4, self.network.n_segm
        self.assertEqual(observed_segm_count, known_segm_count)

    def test_lattice_network_n_node(self):
        known_node_count, observed_node_count = 5, self.network.n_node
        self.assertEqual(observed_node_count, known_node_count)


class TestNetworkTopologyLattice1x1(unittest.TestCase):
    def setUp(self):
        self.network = network_lattice_1x1_no_args

    def test_lattice_network_segm2node(self):
        known_segm2node = {0: [0, 1], 1: [1, 2], 2: [1, 3], 3: [1, 4]}
        observed_segm2node = self.network.segm2node
        self.assertEqual(observed_segm2node, known_segm2node)

    def test_lattice_network_node2segm(self):
        known_node2segm = {0: [0], 1: [0, 1, 2, 3], 2: [1], 3: [2], 4: [3]}
        observed_node2segm = self.network.node2segm
        self.assertEqual(observed_node2segm, known_node2segm)

    def test_lattice_network_segm2segm(self):
        known_segm2segm = {
            0: [1, 2, 3],
            1: [0, 2, 3],
            2: [0, 1, 3],
            3: [0, 1, 2],
        }
        observed_segm2segm = self.network.segm2segm
        self.assertEqual(observed_segm2segm, known_segm2segm)

    def test_lattice_network_node2node(self):
        known_node2node = {0: [1], 1: [0, 2, 3, 4], 2: [1], 3: [1], 4: [1]}
        observed_node2node = self.network.node2node
        self.assertEqual(observed_node2node, known_node2node)


class TestNetworkComponentsLattice1x1(unittest.TestCase):
    def setUp(self):
        # full network
        self.network = network_lattice_2x1x1_all

        # largest component network
        self.network_largest_cc = network_lattice_2x1x1_largest

    def test_lattice_network_segm_components(self):
        known_ccs = {1: [0, 1, 2, 3], 5: [4, 5, 6, 7]}
        observed_ccs = self.network.segm_cc
        self.assertEqual(observed_ccs, known_ccs)

        known_cc_lens = {1: 18.0, 5: 4.0}
        observed_cc_lens = self.network.cc_lens
        self.assertEqual(observed_cc_lens, known_cc_lens)

        known_ccs = 2
        observed_ccs = self.network.n_ccs
        self.assertEqual(observed_ccs, known_ccs)

        known_segms_in_ccs = 8
        observed_segms_in_ccs = self.network.n_segm
        self.assertEqual(observed_segms_in_ccs, known_segms_in_ccs)

    def test_lattice_network_sdata_components(self):
        known_ccs = [1, 1, 1, 1, 5, 5, 5, 5]
        observed_ccs = list(self.network.s_data["CC"])
        self.assertEqual(observed_ccs, known_ccs)

        known_cc_lens = [18.0, 18.0, 18.0, 18.0, 4.0, 4.0, 4.0, 4.0]
        observed_cc_lens = list(self.network.s_data["ccLength"])
        self.assertEqual(observed_cc_lens, known_cc_lens)

    def test_lattice_network_node_components(self):
        known_ccs = {1: [0, 1, 2, 3, 4], 6: [5, 6, 7, 8, 9]}
        observed_ccs = self.network.node_cc
        self.assertEqual(observed_ccs, known_ccs)

    def test_lattice_network_ndata_components(self):
        known_ccs = [1, 1, 1, 1, 1, 6, 6, 6, 6, 6]
        observed_ccs = list(self.network.n_data["CC"])
        self.assertEqual(observed_ccs, known_ccs)

    def test_lattice_network_segm_components_largest(self):
        known_ccs = {1: [0, 1, 2, 3]}
        observed_ccs = self.network_largest_cc.segm_cc
        self.assertEqual(observed_ccs, known_ccs)

        known_cc_lens = {1: 18.0}
        observed_cc_lens = self.network_largest_cc.cc_lens
        self.assertEqual(observed_cc_lens, known_cc_lens)

        known_ccs = 1
        observed_ccs = self.network_largest_cc.n_ccs
        self.assertEqual(observed_ccs, known_ccs)

        known_segms_in_ccs = 4
        observed_segms_in_ccs = self.network_largest_cc.n_segm
        self.assertEqual(observed_segms_in_ccs, known_segms_in_ccs)

    def test_lattice_network_sdata_components_largest(self):
        known_ccs = [1, 1, 1, 1]
        observed_ccs = list(self.network_largest_cc.s_data["CC"])
        self.assertEqual(observed_ccs, known_ccs)

        known_cc_lens = [18.0, 18.0, 18.0, 18.0]
        observed_cc_lens = list(self.network_largest_cc.s_data["ccLength"])
        self.assertEqual(observed_cc_lens, known_cc_lens)

    def test_lattice_network_node_components_largest(self):
        known_ccs = {1: [0, 1, 2, 3, 4]}
        observed_ccs = self.network_largest_cc.node_cc
        self.assertEqual(observed_ccs, known_ccs)

    def test_lattice_network_ndata_components_largest(self):
        known_ccs = [1, 1, 1, 1, 1]
        observed_ccs = list(self.network_largest_cc.n_data["CC"])


class TestNetworkAssociationsLattice1x1(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_lattice_1x1_geomelem)

    def test_lattice_network_segm2geom(self):
        known_type = "LineString"
        observed_type = self.network.segm2geom[0].geom_type
        self.assertEqual(observed_type, known_type)

        known_wkt = "LINESTRING (4.5 0, 4.5 4.5)"
        observed_wkt = self.network.segm2geom[0].wkt
        self.assertEqual(observed_wkt, known_wkt)

    def test_lattice_network_segm2coords(self):
        known_lookup = {
            0: [(4.5, 0.0), (4.5, 4.5)],
            1: [(4.5, 4.5), (4.5, 9.0)],
            2: [(0.0, 4.5), (4.5, 4.5)],
            3: [(4.5, 4.5), (9.0, 4.5)],
        }
        observed_lookup = self.network.segm2coords
        self.assertEqual(observed_lookup, known_lookup)

    def test_lattice_network_node2geom(self):
        known_type = "Point"
        observed_type = self.network.node2geom[0].geom_type
        self.assertEqual(observed_type, known_type)

        known_wkt = "POINT (4.5 0)"
        observed_wkt = self.network.node2geom[0].wkt
        self.assertEqual(observed_wkt, known_wkt)

    def test_lattice_network_node2coords(self):
        known_lookup = {
            0: [(4.5, 0.0)],
            1: [(4.5, 4.5)],
            2: [(4.5, 9.0)],
            3: [(0.0, 4.5)],
            4: [(9.0, 4.5)],
        }
        observed_lookup = self.network.node2coords
        self.assertEqual(observed_lookup, known_lookup)

    def test_lattice_network_length(self):
        known_length, observed_length = 18.0, self.network.network_length
        self.assertEqual(observed_length, known_length)

    def test_lattice_node2degree(self):
        known_node2degree = {0: 1, 1: 4, 2: 1, 3: 1, 4: 1}
        observed_node2degree = self.network.node2degree
        self.assertEqual(observed_node2degree, known_node2degree)

    def test_lattice_ndata_degree(self):
        known_degree = [1, 4, 1, 1, 1]
        observed_degree = list(self.network.n_data["degree"])
        self.assertEqual(observed_degree, known_degree)


class TestNetworkDefineGraphElementsLattice1x1(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_lattice_1x1_geomelem)

    def test_lattice_network_segm2elem(self):
        known_elements = {0: "leaf", 1: "leaf", 2: "leaf", 3: "leaf"}
        observed_elements = self.network.segm2elem
        self.assertEqual(observed_elements, known_elements)

    def test_lattice_network_sdata_segm2elem(self):
        known_elements = ["leaf", "leaf", "leaf", "leaf"]
        observed_elements = list(self.network.s_data["graph_elem"])
        self.assertEqual(observed_elements, known_elements)

    def test_lattice_network_node2elem(self):
        known_elements = {
            0: "leaf",
            1: "branch",
            2: "leaf",
            3: "leaf",
            4: "leaf",
        }
        observed_elements = self.network.node2elem
        self.assertEqual(observed_elements, known_elements)

    def test_lattice_network_ndata_node2elem(self):
        known_elements = ["leaf", "branch", "leaf", "leaf", "leaf"]
        observed_elements = list(self.network.n_data["graph_elem"])
        self.assertEqual(observed_elements, known_elements)


class TestNetworkSimplifyBarb(unittest.TestCase):
    def setUp(self):
        # copy testing
        self.graph = graph_barb
        # inplace
        self.network = network_barb

    def test_simplify_copy_segm2xyid(self):
        known_xyids = {
            0: ["x4.5y0.0", "x4.5y4.5"],
            1: ["x4.5y4.5", "x9.0y4.5", "x9.0y9.0", "x4.5y9.0", "x4.5y4.5"],
            2: ["x0.0y4.5", "x4.5y4.5"],
        }
        observed_xyids = self.graph.segm2xyid
        self.assertEqual(observed_xyids, known_xyids)

    def test_simplify_copy_segm2coords(self):
        known_coords = {
            0: [(4.5, 0.0), (4.5, 4.5)],
            1: [(4.5, 4.5), (9.0, 4.5), (9.0, 9.0), (4.5, 9.0), (4.5, 4.5)],
            2: [(0.0, 4.5), (4.5, 4.5)],
        }
        observed_coords = self.graph.segm2coords
        self.assertEqual(observed_coords, known_coords)

    def test_simplify_copy_segm2elem(self):
        known_elements = {0: "leaf", 1: "branch", 2: "leaf"}
        observed_elements = self.graph.segm2elem
        self.assertEqual(observed_elements, known_elements)

    def test_simplify_copy_segm_cc(self):
        known_ccs = {1: [0, 1, 2]}
        observed_ccs = self.graph.segm_cc
        self.assertEqual(observed_ccs, known_ccs)

    def test_simplify_copy_segm2len(self):
        known_lens = {0: 4.5, 1: 18.0, 2: 4.5}
        observed_lens = self.graph.segm2len
        self.assertEqual(observed_lens, known_lens)

    def test_simplify_copy_node2xyid(self):
        known_xyids = {0: ["x4.5y0.0"], 1: ["x4.5y4.5"], 2: ["x0.0y4.5"]}
        observed_xyids = self.graph.node2xyid
        self.assertEqual(observed_xyids, known_xyids)

    def test_simplify_copy_node2coords(self):
        known_coords = {0: [(4.5, 0.0)], 1: [(4.5, 4.5)], 2: [(0.0, 4.5)]}
        observed_coords = self.graph.node2coords
        self.assertEqual(observed_coords, known_coords)

    def test_simplify_copy_node2elem(self):
        known_elements = {0: "leaf", 1: "branch", 2: "leaf"}
        observed_elements = self.graph.node2elem
        self.assertEqual(observed_elements, known_elements)

    def test_simplify_copy_node_cc(self):
        known_ccs = {1: [0, 1, 2]}
        observed_ccs = self.graph.node_cc
        self.assertEqual(observed_ccs, known_ccs)

    def test_simplify_copy_node2degree(self):
        known_degree = {0: 1, 1: 4, 2: 1}
        observed_degree = self.graph.node2degree
        self.assertEqual(observed_degree, known_degree)

    def test_simplify_inplace_segm2xyid(self):
        known_xyids = {
            0: ["x4.5y0.0", "x4.5y4.5"],
            1: ["x4.5y4.5", "x9.0y4.5", "x9.0y9.0", "x4.5y9.0", "x4.5y4.5"],
            2: ["x0.0y4.5", "x4.5y4.5"],
        }
        observed_xyids = self.network.segm2xyid
        self.assertEqual(observed_xyids, known_xyids)

    def test_simplify_inplace_segm2coords(self):
        known_coords = {
            0: [(4.5, 0.0), (4.5, 4.5)],
            1: [(4.5, 4.5), (9.0, 4.5), (9.0, 9.0), (4.5, 9.0), (4.5, 4.5)],
            2: [(0.0, 4.5), (4.5, 4.5)],
        }
        observed_coords = self.network.segm2coords
        self.assertEqual(observed_coords, known_coords)

    def test_simplify_inplace_segm2elem(self):
        known_elements = {0: "leaf", 1: "branch", 2: "leaf"}
        observed_elements = self.network.segm2elem
        self.assertEqual(observed_elements, known_elements)

    def test_simplify_inplace_segm_cc(self):
        known_ccs = {1: [0, 1, 2]}
        observed_ccs = self.network.segm_cc
        self.assertEqual(observed_ccs, known_ccs)

    def test_simplify_inplace_segm2len(self):
        known_lens = {0: 4.5, 1: 18.0, 2: 4.5}
        observed_lens = self.network.segm2len
        self.assertEqual(observed_lens, known_lens)

    def test_simplify_inplace_node2xyid(self):
        known_xyids = {0: ["x4.5y0.0"], 1: ["x4.5y4.5"], 2: ["x0.0y4.5"]}
        observed_xyids = self.network.node2xyid
        self.assertEqual(observed_xyids, known_xyids)

    def test_simplify_inplace_node2coords(self):
        known_coords = {0: [(4.5, 0.0)], 1: [(4.5, 4.5)], 2: [(0.0, 4.5)]}
        observed_coords = self.network.node2coords
        self.assertEqual(observed_coords, known_coords)

    def test_simplify_inplace_node2elem(self):
        known_elements = {0: "leaf", 1: "branch", 2: "leaf"}
        observed_elements = self.network.node2elem
        self.assertEqual(observed_elements, known_elements)

    def test_simplify_inplace_node_cc(self):
        known_ccs = {1: [0, 1, 2]}
        observed_ccs = self.network.node_cc
        self.assertEqual(observed_ccs, known_ccs)

    def test_simplify_inplace_node2degree(self):
        known_degree = {0: 1, 1: 4, 2: 1}
        observed_degree = self.network.node2degree
        self.assertEqual(observed_degree, known_degree)


if __name__ == "__main__":
    unittest.main()



"""KDTree data testing.
"""

import copy
import unittest
import numpy

from .network_objects import network_lattice_1x1_geomelem
from .network_objects import network_empirical_simplified


class TestKDTreeLattice1x1(unittest.TestCase):
    def setUp(self):
        network = copy.deepcopy(network_lattice_1x1_geomelem)

        # build kd tree
        self.net_nodes_kdtree = network.nodes_kdtree()

        # build kd tree -- from coordinates only
        self.net_nodes_kdtree_coords_only = network.nodes_kdtree(only_coords=True)

    def test_net_nodes_kdtree_data(self):
        known_kdtree_data = numpy.array(
            [[4.5, 0.0], [4.5, 4.5], [4.5, 9.0], [0.0, 4.5], [9.0, 4.5]]
        )
        observed_kdtree_data = self.net_nodes_kdtree.data
        numpy.testing.assert_array_equal(observed_kdtree_data, known_kdtree_data)

    def test_net_nodes_kdtree_query_dists(self):
        known_kdtree_query_dists = numpy.array([0.0, 4.5, 6.36396103])
        observed_kdtree_query_dists = numpy.array(
            self.net_nodes_kdtree.query([4.5, 0.0], k=3)
        )[0, :]
        numpy.testing.assert_array_almost_equal(
            observed_kdtree_query_dists, known_kdtree_query_dists
        )

    def test_net_nodes_kdtree_query_indices(self):
        known_kdtree_query_indices = numpy.array([0.0, 1.0, 3.0])
        observed_kdtree_query_indices = numpy.array(
            self.net_nodes_kdtree.query([4.5, 0.0], k=3)
        )[1, :]
        numpy.testing.assert_array_equal(
            observed_kdtree_query_indices, known_kdtree_query_indices
        )

    def test_net_nodes_kdtree_data_only_coords(self):
        known_kdtree_data = numpy.array(
            [[4.5, 0.0], [4.5, 4.5], [4.5, 9.0], [0.0, 4.5], [9.0, 4.5]]
        )
        observed_kdtree_data = self.net_nodes_kdtree.data
        numpy.testing.assert_array_equal(observed_kdtree_data, known_kdtree_data)

    def test_net_nodes_kdtree_query_dists_only_coords(self):
        known_kdtree_query_dists = numpy.array([0.0, 4.5, 6.36396103])
        observed_kdtree_query_dists = numpy.array(
            self.net_nodes_kdtree.query([4.5, 0.0], k=3)
        )[0, :]
        numpy.testing.assert_array_almost_equal(
            observed_kdtree_query_dists, known_kdtree_query_dists
        )

    def test_net_nodes_kdtree_query_indices_only_coords(self):
        known_kdtree_query_indices = numpy.array([0.0, 1.0, 3.0])
        observed_kdtree_query_indices = numpy.array(
            self.net_nodes_kdtree.query([4.5, 0.0], k=3)
        )[1, :]
        numpy.testing.assert_array_equal(
            observed_kdtree_query_indices, known_kdtree_query_indices
        )


class TestKDTreeEmpirical(unittest.TestCase):
    def setUp(self):
        network = copy.deepcopy(network_empirical_simplified)

        # build kd tree
        self.net_nodes_kdtree = network.nodes_kdtree()

    def test_net_nodes_kdtree_data(self):
        known_kdtree_data = numpy.array(
            [
                [624484.3321678, 165959.33602943],
                [621472.15969302, 164456.34801777],
                [623580.79853327, 164531.84479675],
                [623531.61385791, 164455.13730139],
                [622213.77398256, 166384.29556895],
            ]
        )
        observed_kdtree_data = self.net_nodes_kdtree.data[-5:, :]
        numpy.testing.assert_array_almost_equal(observed_kdtree_data, known_kdtree_data)

    def test_net_nodes_kdtree_query_dists(self):
        known_kdtree_query_dists = numpy.array([0.0, 138.50270336, 369.19792667])
        observed_kdtree_query_dists = numpy.array(
            self.net_nodes_kdtree.query(self.net_nodes_kdtree.data[-1, :], k=3)
        )[0, :]
        numpy.testing.assert_array_almost_equal(
            observed_kdtree_query_dists, known_kdtree_query_dists
        )

    def test_net_nodes_kdtree_query_indices(self):
        known_kdtree_query_indices = numpy.array([285.0, 195.0, 196.0])
        observed_kdtree_query_indices = numpy.array(
            self.net_nodes_kdtree.query(self.net_nodes_kdtree.data[-1, :], k=3)
        )[1, :]
        numpy.testing.assert_array_equal(
            observed_kdtree_query_indices, known_kdtree_query_indices
        )


if __name__ == "__main__":
    unittest.main()



"""Testing errors in tigernet.py
"""


import copy
import geopandas
from shapely.geometry import Point
import unittest

import tigernet
from .. import utils
from .network_objects import network_lattice_1x1_no_args
from .network_objects import network_lattice_1x1_geomelem
from .network_objects import network_empirical_lcc


class TestCostMatrixErrors(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_empirical_lcc)

    def test_non_sequential_ids(self):
        with self.assertRaises(IndexError):
            self.network.cost_matrix()


class TestDataGenerationErrors(unittest.TestCase):
    def setUp(self):
        self.dset = "Edges_Leon_FL_2010"

    def test_read_data_bbox_bad_str(self):
        with self.assertRaises(ValueError):
            tigernet.testing_data(self.dset, bbox="bad_description")

    def test_read_data_bbox_not_iterable(self):
        with self.assertRaises(ValueError):
            tigernet.testing_data(self.dset, bbox=2020)

    def test_read_data_bbox_5_coords(self):
        with self.assertRaises(ValueError):
            tigernet.testing_data(self.dset, bbox=[1.0, 1.0, 2.0, 2.0, 2.0])

    def test_read_data_bbox_4_int_coords(self):
        with self.assertRaises(ValueError):
            tigernet.testing_data(self.dset, bbox=[1, 1, 2, 2])

    def test_read_data_bbox_5_int_coords(self):
        with self.assertRaises(ValueError):
            tigernet.testing_data(self.dset, bbox=[1, 1, 2, 2, 2])


class TestInfoErrors(unittest.TestCase):
    def test_get_discard_segms_bad_year(self):
        with self.assertRaises(KeyError):
            tigernet.get_discard_segms("1500", "12", "073")

    def test_get_discard_segms_no_info(self):
        with self.assertRaises(KeyError):
            tigernet.get_discard_segms("2000", "12", "073")


class TestObservationsErrors(unittest.TestCase):
    def test_no_segm2geom(self):
        network = copy.deepcopy(network_lattice_1x1_no_args)
        with self.assertRaises(AttributeError):
            tigernet.Observations(network, None, None)

    def test_bad_snap_to(self):
        network = copy.deepcopy(network_lattice_1x1_geomelem)
        with self.assertRaises(ValueError):
            tigernet.Observations(network, None, None, snap_to="network")


class TestObs2ObsErrors(unittest.TestCase):
    def test_no_cost_matrix(self):
        network = copy.deepcopy(network_lattice_1x1_geomelem)
        pts = [Point(1, 1), Point(3, 1), Point(1, 3), Point(3, 3)]
        od_data = {"obs_id": ["a", "b", "c", "d"]}
        obs = geopandas.GeoDataFrame(od_data, geometry=pts)
        args = network, obs.copy()
        kwargs = {"df_name": "obs1", "df_key": "obs_id"}
        observations = tigernet.Observations(*args, **kwargs)
        with self.assertRaises(AttributeError):
            tigernet.obs2obs_cost_matrix(observations, network)


class TestStatsErrors(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_lattice_1x1_no_args)

    def test_bad_alpha(self):
        with self.assertRaises(AttributeError):
            self.network.calc_net_stats(conn_stat="alpha")

    def test_bad_stat(self):
        with self.assertRaises(ValueError):
            self.network.calc_net_stats(conn_stat="omega")

    def test_no_ccs_for_alpa(self):
        with self.assertRaises(AttributeError):
            self.network.calc_net_stats(conn_stat="all")

    def test_no_circuity(self):
        with self.assertWarns(UserWarning):
            self.network.calc_net_stats()


class TestUtilsErrors(unittest.TestCase):
    def test_bad_xwalk_column(self):
        with self.assertRaises(ValueError):
            tigernet.utils.xwalk(geopandas.GeoDataFrame(), c1="bad1", c2="bad2")

    def test_assert_2_neighs(self):
        _net = copy.deepcopy(network_lattice_1x1_no_args)
        _net.segm2node[999] = [888, 777, 666]
        with self.assertRaises(AssertionError):
            tigernet.utils.assert_2_neighs(_net)

    def test_bad_branch_or_leaf(self):
        _net = copy.deepcopy(network_lattice_1x1_no_args)
        with self.assertRaises(ValueError):
            tigernet.utils.branch_or_leaf(_net, geom_type="rhombus")


if __name__ == "__main__":
    unittest.main()



def get_discard_mtfcc_by_desc() -> list:
    """Discard these road types from the mtfcc categories."""

    return [
        "Bike Path or Trail",
        "Parking Lot Road",
        "Alley",
        "Vehicular Trail (4WD)",
        "Walkway/Pedestrian Trail",
        "Private Road for service vehicles (logging, oil fields, ranches, etc.)",
    ]


def get_mtfcc_types() -> dict:
    """Return a dictionary of MTFCC road type descriptions.
    See [https://www.census.gov/geo/reference/mtfcc.html] for reference.
    """

    mtfcc_types = {
        "S1100": {
            "FClass": "Primary Road",
            "Desc": "Primary roads are generally divided, "
            + "limited-access highways within the "
            + "interstate highway system or under "
            + "state management, and are "
            + "distinguished by the presence of "
            + "interchanges. These highways are "
            + "accessible by ramps and may include "
            + "some toll highways.",
        },
        "S1200": {
            "FClass": "Secondary Road",
            "Desc": "Secondary roads are main arteries, "
            + "usually in the U.S. Highway, State "
            + "Highway or County Highway system. "
            + "These roads have one or more lanes of "
            + "traffic in each direction, may or may "
            + "not be divided, and usually have "
            + "at-grade intersections with many "
            + "other roads and driveways. They often "
            + "have both a local name and a route "
            + "number.",
        },
        "S1400": {
            "FClass": "Local Neighborhood Road, " + "Rural Road, City Street",
            "Desc": "Generally a paved non-arterial street, "
            + "road, or byway that usually has a "
            + "single lane of traffic in each "
            + "direction. Roads in this feature "
            + "class may be privately or publicly "
            + "maintained. Scenic park roads would "
            + "be included in this feature class, "
            + "as would (depending on the region of "
            + "the country) some unpaved roads.",
        },
        "S1500": {
            "FClass": "Vehicular Trail (4WD)",
            "Desc": "An unpaved dirt trail where a "
            + "four-wheel drive vehicle is required. "
            + "These vehicular trails are found "
            + "almost exclusively in very rural "
            + "areas. Minor, unpaved roads usable by "
            + "ordinary cars and trucks belong in "
            + "the S1400 category.",
        },
        "S1630": {
            "FClass": "Ramp",
            "Desc": "A road that allows controlled access "
            + "from adjacent roads onto a limited "
            + "access highway, often in the form of "
            + "a cloverleaf interchange. These roads "
            + "are unaddressable.",
        },
        "S1640": {
            "FClass": "Service Drive usually along a limited " + "access highway",
            "Desc": "A road, usually paralleling a limited "
            + "access highway, that provides access "
            + "to structures along the highway. "
            + "These roads can be named and may "
            + "intersect with other roads.",
        },
        "S1710": {
            "FClass": "Walkway/Pedestrian Trail",
            "Desc": "A path that is used for walking, being "
            + "either too narrow for or legally "
            + "restricted from vehicular traffic.",
        },
        "S1720": {
            "FClass": "Stairway",
            "Desc": "A pedestrian passageway from one level "
            + "to another by a series of steps.",
        },
        "S1730": {
            "FClass": "Alley",
            "Desc": "A service road that does not generally "
            + "have associated addressed structures "
            + "and is usually unnamed. It is located "
            + "at the rear of buildings and "
            + "properties and is used for "
            + "deliveries.",
        },
        "S1740": {
            "FClass": "Private Road for service vehicles "
            + "(logging, oil fields, ranches, etc.)",
            "Desc": "A road within private property that is "
            + "privately maintained for service, "
            + "extractive, or other purposes. These "
            + "roads are often unnamed.",
        },
        "S1750": {
            "FClass": "Internal U.S. Census Bureau use",
            "Desc": "Internal U.S. Census Bureau use",
        },
        "S1780": {
            "FClass": "Parking Lot Road",
            "Desc": "The main travel route for vehicles "
            + "through a paved parking area.",
        },
        "S1820": {
            "FClass": "Bike Path or Trail",
            "Desc": "A path that is used for manual or "
            + "small, motorized bicycles, being "
            + "either too narrow for or legally "
            + "restricted from vehicular traffic.",
        },
        "S1830": {
            "FClass": "Bridle Path",
            "Desc": "A path that is used for horses, being "
            + "either too narrow for or legally "
            + "restricted from vehicular traffic.",
        },
        "S2000": {
            "FClass": "Road Median",
            "Desc": "The unpaved area or barrier between "
            + "the carriageways of a divided road.",
        },
    }

    return mtfcc_types


def get_discard_segms(year, state_fips, county_fips):
    """This function provides a catalogue of known troublemaker TIGER/Line
    Edges stored at year --> state --> county level. Currently, only
    2010 TIGER/Line Edges for Leon County, Florida are supported.

    Parameters
    ----------
    year : str
        TIGER/Line Edges year.
    state_fips : str
        State FIPS code.
    county_fips : str
        County FIPS code.

    Returns
    -------
    discard : list
        Network segments that are known to be troublemakers.

    """

    catalogue = {
        "2010": {
            "12": {
                "073": [
                    618799725,
                    618799786,
                    618799785,
                    634069404,
                    618799771,
                    618799763,
                    610197711,
                    82844664,
                    82844666,
                    82844213,
                    82844669,
                    82844657,
                    82844670,
                    82844652,
                    82844673,
                ]
            }
        },
        "2000": {"12": {"073": []}},
    }

    try:
        discard = catalogue[year][state_fips][county_fips]
        if not discard:
            raise KeyError
    except KeyError:
        params = year, state_fips, county_fips
        msg = "There was a problem with the query. Check parameters: %s, %s, %s"
        msg += msg % params
        raise KeyError(msg)

    return discard



"""Network objects for testing.
"""

import copy
import tigernet


###############################################################################
############################### Synthetic networks ############################
###############################################################################
# --------------------------------------------------------------------------
# One 1x1 lattice network
#   used in:
#       - test_tigernet_synthetic.TestNetworkBuildLattice1x1
#       - test_tigernet_synthetic.TestNetworkTopologyLattice1x1
#       - test_data_generation.TestObservationDataGenerationSynthetic
#       - test_errors.TestObservationsErrors
#       - test_errors.TestUtilsErrors
#       - test_errors.TestStatsErrors
h1v1 = {"n_hori_lines": 1, "n_vert_lines": 1}
lattice = tigernet.generate_lattice(**h1v1)
network_lattice_1x1_no_args = tigernet.Network(lattice)

# --------------------------------------------------------------------------
# Two 1x1 lattices network (both components)
#   used in:
#       - test_tigernet_synthetic.TestNetworkComponentsLattice1x1
lattice1 = tigernet.generate_lattice(**h1v1)
lattice2 = tigernet.generate_lattice(bounds=[6, 6, 8, 8], **h1v1)
lattice = lattice1.append(lattice2)
lattice.reset_index(drop=True, inplace=True)
kws = {"record_components": True}
network_lattice_2x1x1_all = tigernet.Network(lattice, **kws)

# --------------------------------------------------------------------------
# Two 1x1 lattices network (both components) with a calculated cost matrix (and paths)
#   used in:
#       - test_cost_matrix.TestNetworkCostMatrixLattice1x1_2
network_lattice_2x1x1_wcm_attr = copy.deepcopy(network_lattice_2x1x1_all)
network_lattice_2x1x1_wcm_attr.cost_matrix()
network_lattice_2x1x1_wpaths_attr = copy.deepcopy(network_lattice_2x1x1_all)
network_lattice_2x1x1_wpaths_attr.cost_matrix(wpaths=True)
network_lattice_2x1x1_wcm_var = copy.deepcopy(network_lattice_2x1x1_all).cost_matrix(
    asattr=False
)
_, network_lattice_2x1x1_wpaths_var = copy.deepcopy(
    network_lattice_2x1x1_all
).cost_matrix(wpaths=True, asattr=False)

# --------------------------------------------------------------------------
# Two 1x1 lattices network (largest component)
#   used in:
#       - test_tigernet_synthetic.TestNetworkComponentsLattice1x1
kws.update({"largest_component": True})
network_lattice_2x1x1_largest = tigernet.Network(lattice, **kws)

# --------------------------------------------------------------------------
# One 1x1 lattice network (with recorded geometry and defined graph elements)
#   used in:
#       - test_tigernet_synthetic.TestNetworkAssociationsLattice1x1
#       - test_tigernet_synthetic.TestNetworkDefineGraphElementsLattice1x1
#       - test_kdtree.TestKDTreeLattice1x1
#       - test_errors.TestObservationsErrors
#       - test_observations_synthetic.TestSyntheticObservationsSegmentRandomLattice1x1
#       - test_observations_synthetic.TestSyntheticObservationsNodeRandomLattice1x1
#       - test_observations_synthetic.TestSyntheticObservationsSegmentRandomLattice1x1Restricted
#       - test_observations_synthetic.TestSyntheticObservationsNodeRandomLattice1x1Restricted
lattice = tigernet.generate_lattice(**h1v1)
kws = {"record_geom": True, "def_graph_elems": True}
network_lattice_1x1_geomelem = tigernet.Network(lattice, **kws)

# --------------------------------------------------------------------------
# # One 1x1 lattice network with a calculated cost matrix (and paths)
#   used in:
#       - test_cost_matrix.TestNetworkCostMatrixLattice1x1_1
network_lattice_1x1_wcm_attr = copy.deepcopy(network_lattice_1x1_no_args)
network_lattice_1x1_wcm_attr.cost_matrix()
network_lattice_1x1_wpaths_attr = copy.deepcopy(network_lattice_1x1_no_args)
network_lattice_1x1_wpaths_attr.cost_matrix(wpaths=True)
network_lattice_1x1_wcm_var = copy.deepcopy(network_lattice_1x1_no_args).cost_matrix(
    asattr=False
)
_, network_lattice_1x1_wpaths_var = copy.deepcopy(
    network_lattice_1x1_no_args
).cost_matrix(wpaths=True, asattr=False)

# --------------------------------------------------------------------------
# Copied and simplified inplace barb network
# (with recorded components, recorded geometry and defined graph elements)
#   used in:
#       - test_tigernet_synthetic.TestNetworkSimplifyBarb
barb = tigernet.generate_lattice(wbox=True, **h1v1)
barb = barb[~barb["SegID"].isin([1, 2, 5, 7, 9, 10])]
kws.update({"record_components": True})
network_barb = tigernet.Network(barb, **kws)
# copy
graph_barb = network_barb.simplify_network(**kws)
# inplace
network_barb.simplify_network(inplace=True, **kws)

# --------------------------------------------------------------------------
# Copied and simplified inplace barb network with a calculated cost matrix (and paths)
#   used in:
#       - test_cost_matrix.TestNetworkCostMatrixSimplifyBarb
graph_barb_wcm_copy_attr = copy.deepcopy(graph_barb)
graph_barb_wcm_copy_attr.cost_matrix()
graph_barb_wpaths_copy_attr = copy.deepcopy(graph_barb)
graph_barb_wpaths_copy_attr.cost_matrix(wpaths=True)
graph_barb_wcm_copy_var = copy.deepcopy(graph_barb).cost_matrix(asattr=False)
_, graph_barb_wpaths_copy_var = copy.deepcopy(graph_barb).cost_matrix(
    wpaths=True, asattr=False
)

network_barb_wcm_inplace_attr = copy.deepcopy(network_barb)
network_barb_wcm_inplace_attr.cost_matrix()
network_barb_wpaths_inplace_attr = copy.deepcopy(network_barb)
network_barb_wpaths_inplace_attr.cost_matrix(wpaths=True)
network_barb_wcm_inplace_var = copy.deepcopy(network_barb).cost_matrix(asattr=False)
_, network_barb_wpaths_inplace_var = copy.deepcopy(network_barb).cost_matrix(
    wpaths=True, asattr=False
)

# --------------------------------------------------------------------------
# One 1x1 lattice network with smaller bounds
#   used in:
#       - test_obs2obs_synthetic.TestSyntheticObservationsOrigToXXXXSegments
#       - test_obs2obs_synthetic.TestSyntheticObservationsOrigToXXXXNodes
#       - test_obs2obs_synthetic.TestSyntheticObservationsOrigToDestSegments
#       - test_obs2obs_synthetic.TestSyntheticObservationsOrigToDestNodes
h1v1 = {"n_hori_lines": 1, "n_vert_lines": 1}
lattice = tigernet.generate_lattice(bounds=[0, 0, 4, 4], **h1v1)
network_lattice_1x1_small = tigernet.Network(lattice, **kws)
network_lattice_1x1_small.cost_matrix()


###############################################################################
############################### Empirical networks ############################
###############################################################################


# get the roads shapefile as a GeoDataFrame
gdf = tigernet.testing_data("Edges_Leon_FL_2010")

# filter out only roads
yes_roads = gdf["MTFCC"].str.startswith("S")
roads = gdf[yes_roads].copy()

# Tiger attributes primary and secondary
ATTR1, ATTR2 = "MTFCC", "TLID"

# segment welding and splitting stipulations --------------------------------------------
INTRST = "S1100"  # interstates mtfcc code
RAMP = "S1630"  # ramp mtfcc code
SERV_DR = "S1640"  # service drive mtfcc code
SPLIT_GRP = "FULLNAME"  # grouped by this variable
SPLIT_BY = [RAMP, SERV_DR]  # split interstates by ramps & service
SKIP_RESTR = True  # no weld retry if still MLS

# --------------------------------------------------------------------------
# Full empirical network (all args, all components)
#   used in:
#       - test_tigernet_empirical_gdf.TestNetworkComponentsEmpiricalGDF
discard_segs = None
kwargs = {"from_raw": True, "attr1": ATTR1, "attr2": ATTR2}
comp_kws = {"record_components": True}
kwargs.update(comp_kws)
geom_kws = {"record_geom": True, "calc_len": True}
kwargs.update(geom_kws)
mtfcc_kws = {"discard_segs": discard_segs, "skip_restr": SKIP_RESTR}
mtfcc_kws.update({"mtfcc_split": INTRST, "mtfcc_intrst": INTRST})
mtfcc_kws.update({"mtfcc_split_grp": SPLIT_GRP, "mtfcc_ramp": RAMP})
mtfcc_kws.update({"mtfcc_split_by": SPLIT_BY, "mtfcc_serv": SERV_DR})
kwargs.update(mtfcc_kws)
network_empirical_full = tigernet.Network(roads.copy(), **kwargs)

# --------------------------------------------------------------------------
# Full empirical network (all args, largest component)
#   used in:
#       - test_stats.TestNetworkEntropyEmpirical
#       - test_tigernet_empirical_gdf.TestNetworkBuildEmpiricalGDF
#       - test_tigernet_empirical_gdf.TestNetworkTopologyEmpiricalGDF
#       - test_tigernet_empirical_gdf.TestNetworkComponentsEmpiricalGDF
#       - test_erros.TestCostMatrixErrors
kwargs.update({"largest_component": True, "def_graph_elems": True})
network_empirical_lcc = tigernet.Network(roads.copy(), **kwargs)

# --------------------------------------------------------------------------
# Simplified empirical network (all args, largest component)
#   used in:
#       - test_data_generation.TestObservationDataGenerationSynthetic
#       - test_stats.TestNetworkConnectivityEmpirical
#       - test_tigernet_empirical_gdf.TestNetworkDistanceMetricsEmpiricalGDF
#       - test_tigernet_empirical_gdf.TestNetworkAssociationsEmpiricalGDF
#       - test_tigernet_empirical_gdf.TestNetworkDefineGraphElementsEmpiricalGDF
#       - test_tigernet_empirical_gdf.TestNetworkSimplifyEmpiricalGDF
#       - test_kdtree.TestKDTreeEmpirical
#       - test_observations_synthetic.TestSyntheticObservationsSegmentRandomEmpirical
#       - test_observations_synthetic.TestSyntheticObservationsNodeRandomEmpirical
network_empirical_simplified = copy.deepcopy(network_empirical_lcc)
kws = {"record_components": True, "record_geom": True, "def_graph_elems": True}
# copy
graph_empirical_simplified = network_empirical_simplified.simplify_network(**kws)
# inplace
network_empirical_simplified.simplify_network(inplace=True, **kws)

# --------------------------------------------------------------------------
# Simplified empirical network (all args, largest component) **WITH COST MATRIX**
#   used in:
#       - test_stats.TestNetworkStatsEmpirical
#       - test_stats.TestNetworkDistanceMetricsEmpiricalGDF
#       - test_cost_matrix.TestNetworkCostMatrixEmpircalGDF
#       - test_observations_empirical.TestEmpiricalObservationsOrigToXXXXSegments
#       - test_observations_empirical.TestEmpiricalObservationsOrigToXXXXNodes
network_empirical_simplified_wcm = copy.deepcopy(network_empirical_simplified)
network_empirical_simplified_wcm.cost_matrix(wpaths=True)

# --------------------------------------------------------------------------



"""Testing for tigernet/utils.py
"""

import tigernet
from .. import utils

import copy
import geopandas
import numpy
import operator
import pandas
import unittest
from shapely.geometry import LineString, MultiLineString

from .network_objects import network_lattice_1x1_no_args


class TestUtilsAddLength(unittest.TestCase):
    def test_add_length(self):
        known_length = 18.0
        h1v1 = {"n_hori_lines": 1, "n_vert_lines": 1}
        gdf = tigernet.generate_lattice(**h1v1)
        gdf["length"] = gdf.geometry.length
        gdf = utils.add_length(gdf, len_col="length", geo_col="geometry")
        observed_length = gdf["length"].sum()
        self.assertEqual(observed_length, known_length)


class TestUtilsDropGeoms(unittest.TestCase):
    def test__drop_geoms(self):
        known_values = numpy.array([["c"], ["e"]])
        geom_df = pandas.DataFrame({"points": ["a", "b", "c", "d", "e"]})
        observed_values = utils._drop_geoms(geom_df, [[0, 1], [None, 3]]).values
        numpy.testing.assert_array_equal(observed_values, known_values)


class TestUtilsWeldingFuncs(unittest.TestCase):
    def test__weld_MultiLineString_1(self):
        known_weld_wkt = "LINESTRING (0 0, 0 1, 1 1)"
        mls = MultiLineString((((0, 0), (0, 1)), ((0, 1), (1, 1))))
        observed_weld_wkt = utils._weld_MultiLineString(mls).wkt
        self.assertEqual(observed_weld_wkt, known_weld_wkt)

    def test__weld_MultiLineString_2(self):
        known_weld_wkt = "LINESTRING (0 0, 0 1, 1 1)"
        mls = MultiLineString((((0, 0), (0, 0.99999999)), ((0, 1), (1, 1))))
        observed_weld_wkt = utils._weld_MultiLineString(mls).wkt
        self.assertEqual(observed_weld_wkt, known_weld_wkt)

    def test__weld_MultiLineString_3(self):
        known_weld_wkt = "LINESTRING (1 0, 1 1, 0 1, 0 0)"
        mls = MultiLineString(
            (((0, 1.00000001), (0, 0)), ((1, 1), (0, 1)), ((1, 0), (1, 1)))
        )
        observed_weld_wkt = utils._weld_MultiLineString(mls).wkt
        self.assertEqual(observed_weld_wkt, known_weld_wkt)


class TestUtilsFilterFuncs(unittest.TestCase):
    def setUp(self):
        self.df = pandas.DataFrame().from_dict(
            {"v1": ["x"] * 2 + ["y"] * 2 + ["z"] * 2, "v2": ["a"] * 3 + ["b"] * 3}
        )

    def test_record_filter_mval_in(self):
        known_values = numpy.array([["x", "a"], ["x", "a"], ["y", "a"], ["y", "b"]])
        kws = {"column": "v1", "mval": ["x", "y"], "oper": "in"}
        observed_values = utils.record_filter(self.df.copy(), **kws).values
        numpy.testing.assert_array_equal(observed_values, known_values)

    def test_record_filter_mval_out(self):
        known_values = numpy.array([["z", "b"], ["z", "b"]])
        kws = {"column": "v1", "mval": ["x", "y"], "oper": "out"}
        observed_values = utils.record_filter(self.df.copy(), **kws).values
        numpy.testing.assert_array_equal(observed_values, known_values)

    def test_record_filter_mval_index(self):
        known_values = numpy.array([["y", "b"], ["z", "b"], ["z", "b"]])
        kws = {"column": "index", "mval": [0, 1, 2], "oper": "out"}
        observed_values = utils.record_filter(self.df.copy(), **kws).values
        numpy.testing.assert_array_equal(observed_values, known_values)

    def test_record_filter_sval(self):
        known_values = numpy.array([["x", "a"], ["x", "a"], ["y", "a"]])
        kws = {"column": "v2", "sval": "a", "oper": operator.eq}
        observed_values = utils.record_filter(self.df.copy(), **kws).values
        numpy.testing.assert_array_equal(observed_values, known_values)


class TestUtilRsestrictionWelder(unittest.TestCase):
    def test_restriction_welder_key_error_synth(self):
        known_return = None
        _net = copy.deepcopy(network_lattice_1x1_no_args)
        ATTR1, ATTR2, SPLIT_GRP, SKIP_RESTR = "MTFCC", "TLID", "FULLNAME", True
        INTRST, RAMP, SERV_DR = "S1100", "S1630", "S1640"
        SPLIT_BY = [RAMP, SERV_DR]
        _net.attr1, _net.attr2, _net.skip_restr = ATTR1, ATTR2, SKIP_RESTR
        _net.mtfcc_split, _net.mtfcc_intrst = INTRST, INTRST
        _net.mtfcc_split_grp, _net.mtfcc_ramp = SPLIT_GRP, RAMP
        _net.mtfcc_split_by, _net.mtfcc_serv = SPLIT_BY, SERV_DR
        observed_return = utils.restriction_welder(_net)
        self.assertEqual(observed_return, known_return)


class TestUtilGetLargestCCSNoSmallKeys(unittest.TestCase):
    def test_get_largest_cc_no_small_keys(self):
        known_largest = {1: [0, 1, 2, 3, 4]}
        ccs = {0: [0, 1, 2, 3]}
        ccs.update(known_largest)
        observed_largest = utils.get_largest_cc(ccs, smallkeys=False)
        self.assertEqual(observed_largest, known_largest)


class TestUtilRingCorrection(unittest.TestCase):
    def test_ring_correction_no_correction(self):
        class SynthNetwork:
            def __init__(self):
                self.geo_col = "geometry"
                self.sid_name = "SegID"
                self.xyid = "xyID"

        known_ring_corrections = 0
        net = SynthNetwork()
        line = LineString(((0, 0), (1, 1)))
        ring = LineString(((2, 2), (3, 2), (2.5, 3), (2, 2)))
        gdf = geopandas.GeoDataFrame(geometry=[line, ring])
        gdf["ring"] = ["False", "True"]
        utils.ring_correction(net, gdf.copy())
        observed_ring_corrections = net.corrected_rings
        self.assertEqual(observed_ring_corrections, known_ring_corrections)


class TestUtilGetIntersectingGeoms(unittest.TestCase):
    def test_get_intersecting_geoms_2_dfs_wbool(self):
        class SynthNetwork:
            def __init__(self):
                self.geo_col = "geometry"

        known_len, known_type = 2, tuple
        net = SynthNetwork()
        line = LineString(((0, 0), (1, 1)))
        ring = LineString(((2, 2), (3, 2), (2.5, 3), (2, 2)))
        gdf1 = geopandas.GeoDataFrame(geometry=[line])
        gdf2 = geopandas.GeoDataFrame(geometry=[ring])
        kws = {"df1": gdf1, "geom1": 0, "df2": gdf2, "geom2": 0, "wbool": True}
        observed = utils.get_intersecting_geoms(net, **kws)
        observed_len, observed_type = len(observed), type(observed)
        self.assertEqual(observed_len, known_len)
        self.assertEqual(observed_type, known_type)

    def test_get_intersecting_geoms_2_dfs_xbool(self):
        class SynthNetwork:
            def __init__(self):
                self.geo_col = "geometry"

        known_len, known_type = 0, geopandas.geodataframe.GeoDataFrame
        net = SynthNetwork()
        line = LineString(((0, 0), (1, 1)))
        ring = LineString(((2, 2), (3, 2), (2.5, 3), (2, 2)))
        gdf1 = geopandas.GeoDataFrame(geometry=[line])
        gdf2 = geopandas.GeoDataFrame(geometry=[ring])
        kws = {"df1": gdf1, "geom1": 0, "df2": gdf2, "geom2": 0, "wbool": False}
        observed = utils.get_intersecting_geoms(net, **kws)
        observed_len, observed_type = len(observed), type(observed)
        self.assertEqual(observed_len, known_len)
        self.assertEqual(observed_type, known_type)


class TestUtilSplitLine(unittest.TestCase):
    def setUp(self):
        self.g = "geometry"
        # Case 1: unaltered / Case 1: split
        self.line1, self.line1_idx = LineString(((10, 11), (11, 11))), 0
        self.line2, self.line2_idx = LineString(((10, 10), (10, 12))), 1
        geoms = [self.line1, self.line2]
        rings = ["False", "False"]
        # Case 1: basic ring
        self.line3, self.line3_idx = LineString(((0, 1), (1, 1))), 2
        self.line4 = LineString(((1, 0.5), (2, 0.5), (2, 2), (1, 2), (1, 0.5)))
        self.line4_idx = 3
        geoms += [self.line3, self.line4]
        rings += ["False", "True"]
        # Case 2: horseshoe
        self.line5 = LineString(((0.25, 1), (0.25, 1.5), (0.75, 1.5), (0.75, 1)))
        self.line5_idx = 4
        geoms += [self.line5]
        rings += ["False"]
        # Case 2: s-crossover
        # ---> # line3, line5
        # Case 3
        self.line6, self.line6_idx = LineString(((2.5, 0.5), (2.5, 1.5))), 5
        self.line7, self.line7_idx = LineString(((2.5, 1), (2.5, 2))), 6
        geoms += [self.line6, self.line7]
        rings += ["False", "False"]
        # Case 4
        self.line8, self.line8_idx = LineString(((0, 3), (3, 3))), 7
        self.line9 = LineString(
            ((0.5, 3), (1, 3), (1, 3.5), (2, 3.5), (2, 3), (2.5, 3))
        )
        self.line9_idx = 8
        geoms += [self.line8, self.line9]
        rings += ["False", "False"]
        # Case 5: complex intersection - line & point
        self.line10, self.line10_idx = LineString(((5, 3), (8, 3))), 9
        self.line11 = LineString(((5.5, 3), (6, 3), (6, 3.5), (7, 3.5), (7, 3)))
        self.line11_idx = 10
        geoms += [self.line10, self.line11]
        rings += ["False", "False"]
        # Case 5: complex intersection - line & line
        self.line12, self.line12_idx = LineString(((0, 6), (3, 6))), 11
        self.line13, self.line13_idx = LineString(((0, 7), (3, 7))), 12
        self.line14 = LineString(
            ((0.5, 6), (1, 6), (1, 6.5), (2, 6.5), (2, 7), (2.5, 7))
        )
        self.line14_idx = 13
        geoms += [self.line12, self.line13, self.line14]
        rings += ["False", "False", "False"]
        # Case 5: complex intersection - line & line & point
        self.line15, self.line15_idx = LineString(((5, 6), (8, 6))), 14
        self.line16, self.line16_idx = LineString(((5, 7), (8, 7))), 15
        coords17 = [(5.5, 6), (6, 6), (6, 6.5), (7, 6.5)]
        coords17 += [(7, 7), (7.5, 7), (7.5, 6), (7, 6)]
        self.line17 = LineString(coords17)
        self.line17_idx = 16
        geoms += [self.line15, self.line16, self.line17]
        rings += ["False", "False", "False"]
        # Case 5: complex intersection - line & line & line (camel humps)
        self.line18, self.line18_idx = LineString(((0, 9), (3, 9))), 17
        self.line19 = LineString(
            (
                (0.25, 9),
                (0.5, 9),
                (0.5, 9.5),
                (1, 9.5),
                (1, 9),
                (1.5, 9),
                (1.5, 9.5),
                (2, 9.5),
                (2, 9),
                (2.25, 9),
                (2.25, 8.5),
                (2.75, 8.5),
                (2.75, 9.25),
            )
        )
        self.line19_idx = 17
        geoms += [self.line18, self.line19]
        rings += ["False", "False"]

        self.gdf = geopandas.GeoDataFrame(geometry=geoms)
        self.gdf["ring"] = rings

    def test_split_line_case1_unaltered_line(self):
        known_unaltered_line = self.line1
        idx = self.line1_idx
        loi = self.gdf.loc[idx, self.g]
        args, kwargs = (loi, idx), {"df": self.gdf, "geo_col": self.g}
        observed_unaltered_line = utils._split_line(*args, **kwargs)
        for eidx, xy in enumerate(known_unaltered_line.xy):
            for cidx, coord in enumerate(xy):
                self.assertEqual(observed_unaltered_line[0].xy[eidx][cidx], coord)

    def test_split_line_case1_basic_ring(self):
        known_basic_ring_split = self.line4
        idx = self.line4_idx
        loi = self.gdf.loc[idx, self.g]
        args, kwargs = (loi, idx), {"df": self.gdf, "geo_col": self.g}
        observed_basic_ring_split = utils._split_line(*args, ring_road=True, **kwargs)
        for eidx, xy in enumerate(known_basic_ring_split.xy):
            for cidx, coord in enumerate(xy):
                self.assertEqual(observed_basic_ring_split[0].xy[eidx][cidx], coord)

    def test_split_line_case1_basic_split(self):
        known_basic_split = [
            [[10.0, 10.0], [10.0, 11.0]],
            [[10.0, 10.0], [11.0, 12.0]],
        ]
        idx = self.line2_idx
        loi = self.gdf.loc[idx, self.g]
        args, kwargs = (loi, idx), {"df": self.gdf, "geo_col": self.g}
        observed_basic_split = utils._split_line(*args, **kwargs)
        for lidx, xy in enumerate(known_basic_split):
            for cidx, coord in enumerate(xy):
                self.assertEqual(list(observed_basic_split[lidx].xy[cidx]), coord)

    def test_split_line_case2_horseshoe(self):
        known_horseshoe_split = [[[0.25, 0.25, 0.75, 0.75], [1.0, 1.5, 1.5, 1.0]]]
        idx = self.line5_idx
        loi = self.gdf.loc[idx, self.g]
        args, kwargs = (loi, idx), {"df": self.gdf, "geo_col": self.g}
        observed_horseshoe_split = utils._split_line(*args, **kwargs)
        for lidx, xy in enumerate(known_horseshoe_split):
            for cidx, coord in enumerate(xy):
                self.assertEqual(list(observed_horseshoe_split[lidx].xy[cidx]), coord)

    def test_split_line_case2_standard_multipoint(self):
        known_smp_split = [
            [[0.0, 0.25], [1.0, 1.0]],
            [[0.25, 0.75], [1.0, 1.0]],
            [[0.75, 1.0], [1.0, 1.0]],
        ]
        idx = self.line3_idx
        loi = self.gdf.loc[idx, self.g]
        args, kwargs = (loi, idx), {"df": self.gdf, "geo_col": self.g}
        observed_smp_split = utils._split_line(*args, **kwargs)
        for lidx, xy in enumerate(known_smp_split):
            for cidx, coord in enumerate(xy):
                self.assertEqual(list(observed_smp_split[lidx].xy[cidx]), coord)

    def test_split_line_case3(self):
        known_case3 = [[[2.5, 2.5], [0.5, 1.0]], [[2.5, 2.5], [1.0, 1.5]]]
        idx = self.line6_idx
        loi = self.gdf.loc[idx, self.g]
        args, kwargs = (loi, idx), {"df": self.gdf, "geo_col": self.g}
        observed_case3 = tigernet.utils._split_line(*args, **kwargs)
        for lidx, xy in enumerate(known_case3):
            for cidx, coord in enumerate(xy):
                self.assertEqual(list(observed_case3[lidx].xy[cidx]), coord)

    def test_split_line_case4(self):
        known_case4 = [
            [[0.0, 1.0], [3.0, 3.0]],
            [[1.0, 2.0], [3.0, 3.0]],
            [[2.0, 3.0], [3.0, 3.0]],
        ]
        idx = self.line8_idx
        loi = self.gdf.loc[idx, self.g]
        args, kwargs = (loi, idx), {"df": self.gdf, "geo_col": self.g}
        observed_case4 = tigernet.utils._split_line(*args, **kwargs)
        for lidx, xy in enumerate(known_case4):
            for cidx, coord in enumerate(xy):
                self.assertEqual(list(observed_case4[lidx].xy[cidx]), coord)

    def test_split_line_case51(self):
        known_case51 = [
            [[5.0, 5.5], [3.0, 3.0]],
            [[5.5, 6.0], [3.0, 3.0]],
            [[6.0, 7.0], [3.0, 3.0]],
            [[7.0, 8.0], [3.0, 3.0]],
        ]
        idx = self.line10_idx
        loi = self.gdf.loc[idx, self.g]
        args, kwargs = (loi, idx), {"df": self.gdf, "geo_col": self.g}
        observed_case51 = tigernet.utils._split_line(*args, **kwargs)
        for lidx, xy in enumerate(known_case51):
            for cidx, coord in enumerate(xy):
                self.assertEqual(list(observed_case51[lidx].xy[cidx]), coord)

    def test_split_line_case52(self):
        known_case52 = [
            [[0.5, 1.0, 1.0], [6.0, 6.0, 6.0]],
            [[1.0, 1.0, 2.0, 2.0, 2.0], [6.0, 6.5, 6.5, 7.0, 7.0]],
            [[2.0, 2.5], [7.0, 7.0]],
        ]
        idx = self.line14_idx
        loi = self.gdf.loc[idx, self.g]
        args, kwargs = (loi, idx), {"df": self.gdf, "geo_col": self.g}
        observed_case52 = tigernet.utils._split_line(*args, **kwargs)
        for lidx, xy in enumerate(known_case52):
            for cidx, coord in enumerate(xy):
                self.assertEqual(list(observed_case52[lidx].xy[cidx]), coord)

    def test_split_line_case53(self):
        known_case53 = [
            [[5.5, 6.0, 6.0], [6.0, 6.0, 6.0]],
            [[6.0, 6.0, 7.0, 7.0, 7.5, 7.5, 7.0], [6.0, 6.5, 6.5, 7.0, 7.0, 6.0, 6.0]],
        ]
        idx = self.line17_idx
        loi = self.gdf.loc[idx, self.g]
        args, kwargs = (loi, idx), {"df": self.gdf, "geo_col": self.g}
        observed_case53 = tigernet.utils._split_line(*args, **kwargs)
        for lidx, xy in enumerate(known_case53):
            for cidx, coord in enumerate(xy):
                self.assertEqual(list(observed_case53[lidx].xy[cidx]), coord)

    def test_split_line_case54(self):
        known_case54 = [
            [[0.0, 0.25], [9.0, 9.0]],
            [[0.25, 0.5], [9.0, 9.0]],
            [[0.5, 1.0], [9.0, 9.0]],
            [[1.0, 1.5], [9.0, 9.0]],
            [[1.5, 2.0], [9.0, 9.0]],
            [[2.0, 2.25], [9.0, 9.0]],
            [[2.25, 2.75], [9.0, 9.0]],
            [[2.75, 3.0], [9.0, 9.0]],
        ]
        idx = self.line19_idx
        loi = self.gdf.loc[idx, self.g]
        args, kwargs = (loi, idx), {"df": self.gdf, "geo_col": self.g}
        observed_case54 = tigernet.utils._split_line(*args, **kwargs)
        for lidx, xy in enumerate(known_case54):
            for cidx, coord in enumerate(xy):
                self.assertEqual(list(observed_case54[lidx].xy[cidx]), coord)


if __name__ == "__main__":
    unittest.main()



"""Empirical data testing from a geopandas.GeoDataFrame.
"""

import copy
import unittest
import numpy

from .network_objects import network_empirical_lcc
from .network_objects import network_empirical_full
from .network_objects import graph_empirical_simplified
from .network_objects import network_empirical_simplified


class TestNetworkBuildEmpiricalGDF(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_empirical_lcc)

    def test_network_sdata(self):
        known_segments = 407
        observed_segments = self.network.s_data.shape[0]
        self.assertEqual(observed_segments, known_segments)

        known_length = 74866.5821646358
        observed_length = self.network.s_data.length.sum()
        self.assertAlmostEqual(observed_length, known_length)

    def test_network_ndata(self):
        known_nodes = 348
        observed_nodes = self.network.n_data.shape[0]
        self.assertEqual(observed_nodes, known_nodes)

        known_bounds = numpy.array(
            [
                620989.3023002351,
                163937.37839259504,
                624605.9723871874,
                167048.6931314568,
            ]
        )
        observed_bounds = numpy.array(self.network.n_data.total_bounds)
        numpy.testing.assert_array_almost_equal(observed_bounds, known_bounds)

    def test_network_sdata_ids(self):
        known_ids = [412, 414, 415, 416, 417]
        observed_ids = list(self.network.s_data["SegID"])[-5:]
        self.assertEqual(observed_ids, known_ids)

    def test_network_ndata_ids(self):
        known_ids = [358, 361, 362, 363, 364]
        observed_ids = list(self.network.n_data["NodeID"])[-5:]
        self.assertEqual(observed_ids, known_ids)

    def test_network_segm2xyid(self):
        known_id_xyid = {
            417: [
                "x622213.7739825583y166384.2955689532",
                "x622195.0607060504y166381.4862537613",
                "x622184.5013186845y166381.46491087825",
                "x622181.715678818y166382.34615142326",
                "x622179.9855276581y166383.45124237766",
                "x622167.096623471y166396.0630971515",
                "x622153.7300268554y166407.56541679866",
                "x622145.5590545179y166413.20272313987",
                "x622143.8271135575y166415.1946855663",
                "x622142.3804582829y166418.51752763707",
                "x622140.6237954168y166432.70389406924",
                "x622140.7789228398y166450.88502889618",
                "x622139.3965777882y166469.83907459615",
                "x622137.935323471y166480.3677023711",
                "x622135.714887791y166486.57131120094",
                "x622131.5685194829y166495.76422229825",
            ],
        }
        known_id = list(known_id_xyid.keys())[0]
        known_xyid = list(known_id_xyid.values())[0]
        known_xyid = numpy.array(
            [[float(c) for c in xy[1:].split("y")] for xy in known_xyid]
        )
        observed_id = list(self.network.segm2xyid.keys())[-1]
        observed_xyid = list(self.network.segm2xyid.values())[-1]
        observed_xyid = numpy.array(
            [[float(c) for c in xy[1:].split("y")] for xy in observed_xyid]
        )
        self.assertEqual(observed_id, known_id)
        numpy.testing.assert_array_almost_equal(observed_xyid, known_xyid)

    def test_network_node2xyid(self):
        known_id_xyid = {364: ["x622213.7739825583y166384.2955689532"]}
        known_id = list(known_id_xyid.keys())[0]
        known_xyid = list(known_id_xyid.values())[0]

        observed_id = list(self.network.node2xyid.keys())[-1]
        observed_xyid = list(self.network.node2xyid.values())[-1]

        self.assertEqual(observed_id, known_id)
        self.assertEqual(observed_xyid, known_xyid)

    def test_network_s_ids(self):
        known_ids = [412, 414, 415, 416, 417]
        observed_ids = self.network.s_ids[-5:]
        self.assertEqual(observed_ids, known_ids)

    def test_network_n_ids(self):
        known_ids = [358, 361, 362, 363, 364]
        observed_ids = self.network.n_ids[-5:]
        self.assertEqual(observed_ids, known_ids)

    def test_network_n_segm(self):
        known_segm_count, observed_segm_count = 407, self.network.n_segm
        self.assertEqual(observed_segm_count, known_segm_count)

    def test_network_n_node(self):
        known_node_count, observed_node_count = 348, self.network.n_node
        self.assertEqual(observed_node_count, known_node_count)


class TestNeworkTopologyEmpiricalGDF(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_empirical_lcc)

    def test_network_segm2node(self):
        known_segm2node = {
            0: [0, 1],
            1: [2, 3],
            2: [4, 5],
            3: [5, 6],
            4: [7, 8],
        }
        observed_segm2node = self.network.segm2node
        for known_k, known_v in known_segm2node.items():
            self.assertEqual(observed_segm2node[known_k], known_v)

    def test_network_node2segm(self):
        known_node2segm = {
            0: [0, 5, 9],
            1: [0, 82, 284],
            2: [1, 5, 135, 139, 284],
            3: [1, 7, 8],
            4: [2, 12, 13],
        }
        observed_node2segm = self.network.node2segm
        for known_k, known_v in known_node2segm.items():
            self.assertEqual(observed_node2segm[known_k], known_v)

    def test_network_segm2segm(self):
        known_segm2segm = {
            0: [5, 9, 82, 284],
            1: [5, 7, 8, 135, 139, 284],
            2: [3, 12, 13, 237, 286],
            3: [2, 15, 18, 237, 286],
            4: [10, 139, 162, 295, 348],
        }
        observed_segm2segm = self.network.segm2segm
        for known_k, known_v in known_segm2segm.items():
            self.assertEqual(observed_segm2segm[known_k], known_v)

    def test_network_node2node(self):
        known_node2node = {
            0: [1, 2, 10],
            1: [0, 2, 63],
            2: [0, 1, 3, 7, 195],
            3: [2, 10, 11],
            4: [5, 13, 14],
        }
        observed_node2node = self.network.node2node
        for known_k, known_v in known_node2node.items():
            self.assertEqual(observed_node2node[known_k], known_v)


class TestNeworkComponentsEmpiricalGDF(unittest.TestCase):
    def setUp(self):
        # full network
        self.network = copy.deepcopy(network_empirical_full)
        # largest component network
        self.network_largest_cc = copy.deepcopy(network_empirical_lcc)

    def test_network_segm_components(self):
        known_ccs = {106: [105, 106, 108, 110, 263, 264]}
        observed_ccs = self.network.segm_cc
        for known_k, known_v in known_ccs.items():
            self.assertEqual(observed_ccs[known_k], known_v)

        known_cc_lens = {
            74: 74866.5821646358,
            106: 771.0198272868483,
            166: 245.73023272135015,
            172: 22.726003850972294,
            312: 159.71061675074486,
            394: 289.03754534132827,
            413: 41.69245093932556,
        }
        observed_cc_lens = self.network.cc_lens
        for k, v in known_cc_lens.items():
            self.assertAlmostEqual(observed_cc_lens[k], v)

        known_ccs = 7
        observed_ccs = self.network.n_ccs
        self.assertEqual(observed_ccs, known_ccs)

        known_segms_in_ccs = 418
        observed_segms_in_ccs = self.network.n_segm
        self.assertEqual(observed_segms_in_ccs, known_segms_in_ccs)

    def test_network_sdata_components(self):
        known_ccs = [74, 74, 74, 74, 74, 106, 106, 74, 106, 74]
        observed_ccs = list(self.network.s_data["CC"])[100:110]
        self.assertEqual(observed_ccs, known_ccs)

        known_cc_lens = [771.0198272868483, 74866.5821646358]
        observed_cc_lens = list(self.network.s_data["ccLength"])[108:110]
        for k, o in zip(known_cc_lens, observed_cc_lens):
            self.assertAlmostEqual(o, k)

    def test_network_node_components(self):
        known_ccs = {159: [158, 159, 160, 163, 164, 298, 299]}
        observed_ccs = self.network.node_cc
        for known_k, known_v in known_ccs.items():
            self.assertEqual(observed_ccs[known_k], known_v)

    def test_network_ndata_components(self):
        known_ccs = [360, 30, 30, 30, 30]
        observed_ccs = list(self.network.n_data["CC"])[360:]
        self.assertEqual(observed_ccs, known_ccs)

    def test_network_segm_components_largest(self):
        known_ccs = {74: [0, 1, 2, 3, 4]}
        observed_ccs = self.network_largest_cc.segm_cc
        for known_k, known_v in known_ccs.items():
            self.assertEqual(observed_ccs[known_k][:5], known_v)

        known_ccs_k, known_cc_lens = 74, 74866.5821646358
        observed_cc_lens = self.network_largest_cc.cc_lens
        self.assertAlmostEqual(observed_cc_lens[known_ccs_k], known_cc_lens)

        known_ccs = 1
        observed_ccs = self.network_largest_cc.n_ccs
        self.assertEqual(observed_ccs, known_ccs)

        known_segms_in_ccs = 407
        observed_segms_in_ccs = self.network_largest_cc.n_segm
        self.assertEqual(observed_segms_in_ccs, known_segms_in_ccs)

    def test_network_sdata_components_largest(self):
        known_ccs = [74, 74, 74, 74, 74]
        observed_ccs = list(self.network_largest_cc.s_data["CC"])[:5]
        self.assertEqual(observed_ccs, known_ccs)

        known_cc_lens = [74866.5821646358, 74866.5821646358]
        observed_cc_lens = list(self.network_largest_cc.s_data["ccLength"])[:2]
        for k, o in zip(known_cc_lens, observed_cc_lens):
            self.assertAlmostEqual(o, k)

    def test_network_node_components_largest(self):
        known_ccs = {30: [0, 1, 2, 3, 4]}
        observed_ccs = self.network_largest_cc.node_cc
        for known_k, known_v in known_ccs.items():
            self.assertEqual(observed_ccs[known_k][:5], known_v)

    def test_network_ndata_components_largest(self):
        known_ccs = [30, 30, 30, 30, 30]
        observed_ccs = list(self.network_largest_cc.n_data["CC"])[:5]


class TestNetworkAssociationsEmpiricalGDF(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_empirical_lcc)

    def test_network_segm2geom(self):
        known_type = "LineString"
        observed_type = self.network.segm2geom[0].geom_type
        self.assertEqual(observed_type, known_type)

        known_wkt = "LINESTRING (623605.9583105363 166441.9652821319, 623642.2579218673 166435.6136040619, 623661.2704885595 166432.9939635286, 623683.5540714423 166427.0556520971, 623703.2557014348 166416.5666028635, 623719.0358090349 166399.5284506257, 623728.2024647847 166377.8199573702, 623732.1909850998 166353.5505257068, 623732.1809754729 166313.9739520327)"
        observed_wkt = self.network.segm2geom[0].wkt[:20]
        self.assertEqual(observed_wkt, known_wkt[:20])

    def test_network_segm2coords(self):
        known_coords = numpy.array(
            [
                (622213.7739825583, 166384.2955689532),
                (622195.0607060504, 166381.4862537613),
                (622184.5013186845, 166381.46491087825),
                (622181.715678818, 166382.34615142326),
                (622179.9855276581, 166383.45124237766),
                (622167.096623471, 166396.0630971515),
                (622153.7300268554, 166407.56541679866),
                (622145.5590545179, 166413.20272313987),
                (622143.8271135575, 166415.1946855663),
                (622142.3804582829, 166418.51752763707),
                (622140.6237954168, 166432.70389406924),
                (622140.7789228398, 166450.88502889618),
                (622139.3965777882, 166469.83907459615),
                (622137.935323471, 166480.3677023711),
                (622135.714887791, 166486.57131120094),
                (622131.5685194829, 166495.76422229825),
            ],
        )

        observed_coords = numpy.array(self.network.segm2coords[417])
        numpy.testing.assert_array_almost_equal(observed_coords, known_coords)

    def test_network_node2geom(self):
        known_type = "Point"
        observed_type = self.network.node2geom[0].geom_type
        self.assertEqual(observed_type, known_type)

        known_wkt = "POINT (623605.9583105363 166441.9652821319)"
        observed_wkt = self.network.node2geom[0].wkt[:20]
        self.assertEqual(observed_wkt, known_wkt[:20])

    def test_network_node2coords(self):
        known_coords = numpy.array([(622213.7739825583, 166384.2955689532)])
        observed_coords = numpy.array(self.network.node2coords[364])
        numpy.testing.assert_array_almost_equal(observed_coords, known_coords)

    def test_network_length(self):
        known_length, observed_length = 74866.58216463577, self.network.network_length
        self.assertAlmostEqual(observed_length, known_length)

    def test_node2degree(self):
        known_node2degree = [(100, 3), (101, 3), (102, 2), (103, 3), (104, 3)]
        observed_node2degree = list(self.network.node2degree.items())[100:105]
        self.assertEqual(observed_node2degree, known_node2degree)

    def test_ndata_degree(self):
        known_degree = [3, 3, 5, 3, 3, 4, 3, 3, 4, 1, 3, 2, 1, 3, 1]
        observed_degree = list(self.network.n_data["degree"])[:15]
        self.assertEqual(observed_degree, known_degree)


class TestNetworkDefineGraphElementsEmpiricalGDF(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_empirical_lcc)

    def test_network_segm2elem(self):
        known_element_keys = [414, 415, 416, 417]
        known_element_values = ["leaf", "leaf", "leaf", "leaf"]
        observed_element_keys = list(self.network.segm2elem.keys())[-4:]
        observed_element_values = list(self.network.segm2elem.values())[-4:]
        self.assertEqual(observed_element_keys, known_element_keys)
        self.assertEqual(observed_element_values, known_element_values)

    def test_network_sdata_segm2elem(self):
        known_elements = ["leaf", "leaf", "leaf", "leaf"]
        observed_elements = list(self.network.s_data["graph_elem"])[-4:]
        self.assertEqual(observed_elements, known_elements)

    def test_network_node2elem(self):
        known_element_keys = [361, 362, 363, 364]
        known_element_values = ["leaf", "leaf", "leaf", "leaf"]
        observed_element_keys = list(self.network.node2elem.keys())[-4:]
        observed_element_values = list(self.network.node2elem.values())[-4:]
        self.assertEqual(observed_element_keys, known_element_keys)
        self.assertEqual(observed_element_values, known_element_values)

    def test_network_ndata_node2elem(self):
        known_elements = ["leaf", "leaf", "leaf", "leaf"]
        observed_elements = list(self.network.n_data["graph_elem"])[-4:]
        self.assertEqual(observed_elements, known_elements)


class TestNetworkSimplifyEmpiricalGDF(unittest.TestCase):
    def setUp(self):
        # copy testing
        self.graph = copy.deepcopy(graph_empirical_simplified)

        # inplace
        self.network = copy.deepcopy(network_empirical_simplified)

    def test_simplify_copy_segm2xyid(self):
        known_id = 344
        known_xyid = [
            "x622213.7739825583y166384.2955689532",
            "x622195.0607060504y166381.4862537613",
            "x622184.5013186845y166381.46491087825",
            "x622181.715678818y166382.34615142326",
            "x622179.9855276581y166383.45124237766",
            "x622167.096623471y166396.0630971515",
            "x622153.7300268554y166407.56541679866",
            "x622145.5590545179y166413.20272313987",
            "x622143.8271135575y166415.1946855663",
            "x622142.3804582829y166418.51752763707",
            "x622140.6237954168y166432.70389406924",
            "x622140.7789228398y166450.88502889618",
            "x622139.3965777882y166469.83907459615",
            "x622137.935323471y166480.3677023711",
            "x622135.714887791y166486.57131120094",
            "x622131.5685194829y166495.76422229825",
        ]
        known_xyid = numpy.array(
            [[float(c) for c in xy[1:].split("y")] for xy in known_xyid]
        )
        observed_xyid = self.graph.segm2xyid[known_id]
        observed_xyid = numpy.array(
            [[float(c) for c in xy[1:].split("y")] for xy in observed_xyid]
        )
        numpy.testing.assert_array_almost_equal(observed_xyid, known_xyid)

    def test_simplify_copy_segm2coords(self):
        known_id = 344
        known_coords = [
            (622213.7739825583, 166384.2955689532),
            (622195.0607060504, 166381.4862537613),
            (622184.5013186845, 166381.46491087825),
            (622181.715678818, 166382.34615142326),
            (622179.9855276581, 166383.45124237766),
            (622167.096623471, 166396.0630971515),
            (622153.7300268554, 166407.56541679866),
            (622145.5590545179, 166413.20272313987),
            (622143.8271135575, 166415.1946855663),
            (622142.3804582829, 166418.51752763707),
            (622140.6237954168, 166432.70389406924),
            (622140.7789228398, 166450.88502889618),
            (622139.3965777882, 166469.83907459615),
            (622137.935323471, 166480.3677023711),
            (622135.714887791, 166486.57131120094),
            (622131.5685194829, 166495.76422229825),
        ]
        observed_coords = self.graph.segm2coords[known_id]
        numpy.testing.assert_array_almost_equal(observed_coords, known_coords)

    def test_simplify_copy_segm2elem(self):
        known_elements = {342: "leaf", 343: "leaf", 344: "leaf"}
        observed_elements = self.graph.segm2elem
        for k, v in known_elements.items():
            self.assertEqual(observed_elements[k], v)

    def test_simplify_copy_segm_cc(self):
        known_root, known_ccs = 64, [340, 341, 342, 343, 344]
        observed_ccs = self.graph.segm_cc[known_root][-5:]
        self.assertEqual(observed_ccs, known_ccs)

    def test_simplify_copy_segm2len(self):
        known_id, known_len = 344, 165.09903569556914
        observed_len = self.graph.segm2len[known_id]
        self.assertAlmostEqual(observed_len, known_len)

    def test_simplify_copy_node2xyid(self):
        known_id, known_xyid = 285, ["x622213.7739825583y166384.2955689532"]
        known_xyid = numpy.array(
            [[float(c) for c in xy[1:].split("y")] for xy in known_xyid]
        )
        observed_xyid = self.graph.node2xyid[known_id]
        observed_xyid = numpy.array(
            [[float(c) for c in xy[1:].split("y")] for xy in observed_xyid]
        )
        numpy.testing.assert_array_almost_equal(observed_xyid, known_xyid)

    def test_simplify_copy_node2coords(self):
        known_id, known_coords = 285, [(622213.7739825583, 166384.2955689532)]
        known_xyid = numpy.array(known_coords)
        observed_coords = numpy.array(self.graph.node2coords[known_id])
        numpy.testing.assert_array_almost_equal(observed_coords, known_coords)

    def test_simplify_copy_node2elem(self):
        known_elements = {283: "leaf", 284: "leaf", 285: "leaf"}
        observed_elements = self.graph.node2elem
        for k, v in known_elements.items():
            self.assertEqual(observed_elements[k], v)

    def test_simplify_copy_node_cc(self):
        known_root, known_ccs = 29, [281, 282, 283, 284, 285]
        observed_ccs = self.graph.node_cc[known_root]
        self.assertEqual(observed_ccs[-5:], known_ccs)

    def test_simplify_copy_node2degree(self):
        known_degree = [(0, 3), (1, 3), (2, 5), (3, 3)]
        observed_degree = list(self.graph.node2degree.items())[:4]
        self.assertEqual(observed_degree, known_degree)

    def test_simplify_inplace_segm2xyid(self):
        known_id = 344
        known_xyid = [
            "x622213.7739825583y166384.2955689532",
            "x622195.0607060504y166381.4862537613",
            "x622184.5013186845y166381.46491087825",
            "x622181.715678818y166382.34615142326",
            "x622179.9855276581y166383.45124237766",
            "x622167.096623471y166396.0630971515",
            "x622153.7300268554y166407.56541679866",
            "x622145.5590545179y166413.20272313987",
            "x622143.8271135575y166415.1946855663",
            "x622142.3804582829y166418.51752763707",
            "x622140.6237954168y166432.70389406924",
            "x622140.7789228398y166450.88502889618",
            "x622139.3965777882y166469.83907459615",
            "x622137.935323471y166480.3677023711",
            "x622135.714887791y166486.57131120094",
            "x622131.5685194829y166495.76422229825",
        ]
        known_xyid = numpy.array(
            [[float(c) for c in xy[1:].split("y")] for xy in known_xyid]
        )
        observed_xyid = self.network.segm2xyid[known_id]
        observed_xyid = numpy.array(
            [[float(c) for c in xy[1:].split("y")] for xy in observed_xyid]
        )
        numpy.testing.assert_array_almost_equal(observed_xyid, known_xyid)

    def test_simplify_inplace_segm2coords(self):
        known_id = 344
        known_coords = [
            (622213.7739825583, 166384.2955689532),
            (622195.0607060504, 166381.4862537613),
            (622184.5013186845, 166381.46491087825),
            (622181.715678818, 166382.34615142326),
            (622179.9855276581, 166383.45124237766),
            (622167.096623471, 166396.0630971515),
            (622153.7300268554, 166407.56541679866),
            (622145.5590545179, 166413.20272313987),
            (622143.8271135575, 166415.1946855663),
            (622142.3804582829, 166418.51752763707),
            (622140.6237954168, 166432.70389406924),
            (622140.7789228398, 166450.88502889618),
            (622139.3965777882, 166469.83907459615),
            (622137.935323471, 166480.3677023711),
            (622135.714887791, 166486.57131120094),
            (622131.5685194829, 166495.76422229825),
        ]
        observed_coords = self.network.segm2coords[known_id]
        numpy.testing.assert_array_almost_equal(observed_coords, known_coords)

    def test_simplify_inplace_segm2elem(self):
        known_elements = {342: "leaf", 343: "leaf", 344: "leaf"}
        observed_elements = self.network.segm2elem
        for k, v in known_elements.items():
            self.assertEqual(observed_elements[k], v)

    def test_simplify_inplace_segm_cc(self):
        known_root, known_ccs = 64, [340, 341, 342, 343, 344]
        observed_ccs = self.network.segm_cc[known_root][-5:]
        self.assertEqual(observed_ccs, known_ccs)

    def test_simplify_inplace_segm2len(self):
        known_id, known_len = 344, 165.09903569556914
        observed_len = self.network.segm2len[known_id]
        self.assertAlmostEqual(observed_len, known_len)

    def test_simplify_inplace_node2xyid(self):
        known_id, known_xyid = 285, ["x622213.7739825583y166384.2955689532"]
        known_xyid = numpy.array(
            [[float(c) for c in xy[1:].split("y")] for xy in known_xyid]
        )
        observed_xyid = self.network.node2xyid[known_id]
        observed_xyid = numpy.array(
            [[float(c) for c in xy[1:].split("y")] for xy in observed_xyid]
        )
        numpy.testing.assert_array_almost_equal(observed_xyid, known_xyid)

    def test_simplify_inplace_node2coords(self):
        known_id, known_coords = 285, [(622213.7739825583, 166384.2955689532)]
        known_xyid = numpy.array(known_coords)
        observed_coords = numpy.array(self.network.node2coords[known_id])
        numpy.testing.assert_array_almost_equal(observed_coords, known_coords)

    def test_simplify_inplace_node2elem(self):
        known_elements = {283: "leaf", 284: "leaf", 285: "leaf"}
        observed_elements = self.network.node2elem
        for k, v in known_elements.items():
            self.assertEqual(observed_elements[k], v)

    def test_simplify_inplace_node_cc(self):
        known_root, known_ccs = 29, [281, 282, 283, 284, 285]
        observed_ccs = self.network.node_cc[known_root]
        self.assertEqual(observed_ccs[-5:], known_ccs)

    def test_simplify_inplace_node2degree(self):
        known_degree = [(0, 3), (1, 3), (2, 5), (3, 3)]
        observed_degree = list(self.network.node2degree.items())[:4]
        self.assertEqual(observed_degree, known_degree)


if __name__ == "__main__":
    unittest.main()



"""Data generation for testing for tigernet.py
"""

import unittest
import numpy

import tigernet
from .network_objects import network_lattice_1x1_no_args
from .network_objects import network_empirical_simplified


class TestNetworkDataGenerationCreation(unittest.TestCase):
    def setUp(self):
        b1133, nh, nv = [-1, -1, 3, 3], 3, 4
        self._34_kws = {"bounds": b1133, "n_hori_lines": nh, "n_vert_lines": nv}

    def test_generate_sine_lines(self):
        sine = tigernet.generate_sine_lines()
        observed_length = sine.loc[(sine["SegID"] == 0), "geometry"].squeeze().length
        known_length = 11.774626215766602
        self.assertAlmostEqual(observed_length, known_length, 10)

    def test_generate_lattice_2x2_xbox_0099(self):
        lat = tigernet.generate_lattice()
        observed_length = lat.length.sum()
        known_length = 36.0
        self.assertEqual(observed_length, known_length)

    def test_generate_lattice_2x2_wbox_0099(self):
        lat = tigernet.generate_lattice(wbox=True)
        observed_length = lat.length.sum()
        known_length = 72.0
        self.assertEqual(observed_length, known_length)

    def test_generate_lattice_3x4_xbox_neg1neg133(self):
        lat = tigernet.generate_lattice(**self._34_kws)
        observed_length = lat.length.sum()
        known_length = 27.8
        self.assertEqual(observed_length, known_length)

    def test_generate_lattice_2x2_wbox_neg1neg133(self):
        lat = tigernet.generate_lattice(wbox=True, **self._34_kws)
        observed_length = lat.length.sum()
        known_length = 44.2
        self.assertEqual(observed_length, known_length)


class TestNetworkDataGenerationRead(unittest.TestCase):
    def setUp(self):
        self.dset = "Edges_Leon_FL_2010"

    def test_good_read_no_bbox(self):
        known_recs = 37798
        bbox = None
        observed_recs = tigernet.testing_data(self.dset, bbox=bbox).shape[0]
        self.assertEqual(observed_recs, known_recs)

    def test_good_read_from_coords(self):
        known_recs = 4
        bbox = (-84.34, 30.4935, -84.3378, 30.494)
        observed_recs = tigernet.testing_data(self.dset, bbox=bbox).shape[0]
        self.assertEqual(observed_recs, known_recs)


class TestObservationDataGenerationSynthetic(unittest.TestCase):
    def setUp(self):
        self.network = network_lattice_1x1_no_args

    def test_generate_observations_totalbounds(self):
        n_obs = 5
        obs = tigernet.generate_obs(n_obs, self.network.s_data)
        known_n_obs = n_obs
        observed_n_obs = obs.shape[0]
        self.assertEqual(observed_n_obs, known_n_obs)

        known_coords = numpy.array(
            [
                (4.939321535345923, 6.436704297351775),
                (5.4248703846447945, 4.903948646972072),
                (3.8128931940501425, 5.813047017599905),
                (3.9382849013642325, 8.025957007038718),
                (8.672964844509263, 3.4509736694319995),
            ]
        )
        observed_coords = numpy.array([(p.x, p.y) for p in obs.geometry])
        numpy.testing.assert_array_almost_equal(observed_coords, known_coords)

    def test_generate_observations_inbuffer(self):
        n_obs = 5
        obs = tigernet.generate_obs(n_obs, self.network.s_data, near_net=0.5)
        known_n_obs = n_obs
        observed_n_obs = obs.shape[0]
        self.assertEqual(observed_n_obs, known_n_obs)

        known_coords = numpy.array(
            [
                (4.939321535345923, 6.436704297351775),
                (5.4248703846447945, 4.903948646972072),
                (7.125525342743981, 4.76005427777614),
                (4.153314260276387, 7.024762586578099),
                (4.696634895750645, 3.731957459914712),
            ]
        )
        observed_coords = numpy.array([(p.x, p.y) for p in obs.geometry])
        numpy.testing.assert_array_almost_equal(observed_coords, known_coords)


class TestObservationDataGenerationEmpirical(unittest.TestCase):
    def setUp(self):
        self.network = network_empirical_simplified

    def test_generate_observations_totalbounds(self):

        n_obs = 500
        obs = tigernet.generate_obs(n_obs, self.network.s_data)
        known_n_obs = n_obs
        observed_n_obs = obs.shape[0]
        self.assertEqual(observed_n_obs, known_n_obs)

        known_coords = numpy.array(
            [
                (622974.1796832045, 166162.55760926675),
                (623169.2985719838, 165632.6814708112),
                (622521.5219401979, 165946.95826632337),
                (622571.9108776418, 166711.9648736473),
                (624474.552580049, 165130.38564160923),
            ]
        )
        observed_coords = numpy.array([(p.x, p.y) for p in obs.geometry])[:5]
        numpy.testing.assert_array_almost_equal(observed_coords, known_coords)

    def test_generate_observations_inbuffer(self):
        n_obs = 500
        obs = tigernet.generate_obs(n_obs, self.network.s_data, near_net=30)
        known_n_obs = n_obs
        observed_n_obs = obs.shape[0]
        self.assertEqual(observed_n_obs, known_n_obs)

        known_coords = numpy.array(
            [
                (622571.9108776418, 166711.9648736473),
                (624474.552580049, 165130.38564160923),
                (623803.6385443554, 166644.26001242414),
                (622876.6555154349, 165227.52219813256),
                (623400.7775349629, 166023.94388077687),
            ]
        )
        observed_coords = numpy.array([(p.x, p.y) for p in obs.geometry])[:5]
        numpy.testing.assert_array_almost_equal(observed_coords, known_coords)

    def test_generate_observations_inbuffer_restrict(self):
        n_obs = 500
        obs = tigernet.generate_obs(
            n_obs, self.network.s_data, restrict=("MTFCC", ["S1100"]), near_net=30
        )
        known_n_obs = n_obs
        observed_n_obs = obs.shape[0]
        self.assertEqual(observed_n_obs, known_n_obs)

        known_coords = numpy.array(
            [
                (622571.9108776418, 166711.9648736473),
                (624474.552580049, 165130.38564160923),
                (623803.6385443554, 166644.26001242414),
                (622876.6555154349, 165227.52219813256),
                (623400.7775349629, 166023.94388077687),
            ]
        )
        observed_coords = numpy.array([(p.x, p.y) for p in obs.geometry])[:5]
        numpy.testing.assert_array_almost_equal(observed_coords, known_coords)


if __name__ == "__main__":
    unittest.main()



"""Testing for tigernet.py
"""

import copy
import unittest
import numpy
import geopandas
from shapely.geometry import LineString


import tigernet
from .network_objects import network_empirical_lcc
from .network_objects import network_empirical_simplified
from .network_objects import network_empirical_simplified_wcm


##########################################################################################
# Synthetic testing
##########################################################################################


class TestNetworkStatsBarb(unittest.TestCase):
    def setUp(self):
        lat = tigernet.generate_lattice(n_hori_lines=1, n_vert_lines=1, wbox=True)
        lat = lat[~lat["SegID"].isin([3, 5, 8])]
        rec = {
            "geometry": LineString(((4.5, 9), (4.5, 13.5))),
            "SegID": 13,
            "MTFCC": "S1400",
        }
        barb = lat.append(rec, ignore_index=True)

        # full network
        self.network = tigernet.Network(s_data=barb.copy())
        with self.assertWarns(UserWarning):
            self.network.calc_net_stats()

        # simplified network
        self.graph = self.network.simplify_network()
        with self.assertWarns(UserWarning):
            self.graph.calc_net_stats()

    def test_barb_network_sinuosity(self):
        known_sinuosity = [1.0] * 10
        observed_sinuosity = list(self.network.s_data["sinuosity"])
        self.assertEqual(observed_sinuosity, known_sinuosity)

    def test_barb_network_sinuosity_stats(self):
        known_max = 1.0
        observed_max = self.network.max_sinuosity
        self.assertEqual(observed_max, known_max)

        known_min = 1.0
        observed_min = self.network.min_sinuosity
        self.assertEqual(observed_min, known_min)

        known_mean = 1.0
        observed_mean = self.network.mean_sinuosity
        self.assertEqual(observed_mean, known_mean)

        known_std = 0.0
        observed_std = self.network.std_sinuosity
        self.assertEqual(observed_std, known_std)

    def test_barb_graph_sinuosity(self):
        known_sinuosity = [2.23606797749979, numpy.inf, 1.0]
        observed_sinuosity = list(self.graph.s_data["sinuosity"])
        self.assertEqual(observed_sinuosity, known_sinuosity)

    def test_barb_graph_sinuosity_stats(self):
        known_max = 2.23606797749979
        observed_max = self.graph.max_sinuosity
        self.assertEqual(observed_max, known_max)

        known_min = 1.0
        observed_min = self.graph.min_sinuosity
        self.assertEqual(observed_min, known_min)

        known_mean = 1.8240453183331933
        observed_mean = self.graph.mean_sinuosity
        self.assertEqual(observed_mean, known_mean)

        known_std = 0.71364417954618
        observed_std = self.graph.std_sinuosity
        self.assertEqual(observed_std, known_std)

    def test_barb_network_node_degree_stats(self):
        known_max = 4
        observed_max = self.network.max_node_degree
        self.assertEqual(observed_max, known_max)

        known_min = 1
        observed_min = self.network.min_node_degree
        self.assertEqual(observed_min, known_min)

        known_mean = 2.0
        observed_mean = self.network.mean_node_degree
        self.assertEqual(observed_mean, known_mean)

        known_std = 0.816496580927726
        observed_std = self.network.std_node_degree
        self.assertEqual(observed_std, known_std)

    def test_barb_graph_node_degree_stats(self):
        known_max = 4
        observed_max = self.graph.max_node_degree
        self.assertEqual(observed_max, known_max)

        known_min = 1
        observed_min = self.graph.min_node_degree
        self.assertEqual(observed_min, known_min)

        known_mean = 2.0
        observed_mean = self.graph.mean_node_degree
        self.assertEqual(observed_mean, known_mean)

        known_std = 1.7320508075688772
        observed_std = self.graph.std_node_degree
        self.assertEqual(observed_std, known_std)


class TestNetworkStatsSineLine(unittest.TestCase):
    def setUp(self):
        sine = tigernet.generate_sine_lines()
        sine = tigernet.generate_sine_lines()
        sine = sine[sine["SegID"].isin([0, 1, 2, 3])]

        # full network
        self.network = tigernet.Network(s_data=sine.copy())
        with self.assertWarns(UserWarning):
            self.network.calc_net_stats()

        # simplified network
        self.graph = self.network.simplify_network()
        with self.assertWarns(UserWarning):
            self.graph.calc_net_stats()

    def test_sine_network_sinuosity(self):
        known_sinuosity = [
            1.1913994275103448,
            1.0377484765201541,
            1.0714252226602858,
            1.1885699897294775,
        ]
        observed_sinuosity = list(self.network.s_data["sinuosity"])
        self.assertEqual(observed_sinuosity, known_sinuosity)

    def test_sine_network_sinuosity_stats(self):
        known_max = 1.1913994275103448
        observed_max = self.network.max_sinuosity
        self.assertEqual(observed_max, known_max)

        known_min = 1.0377484765201541
        observed_min = self.network.min_sinuosity
        self.assertEqual(observed_min, known_min)

        known_mean = 1.1222857791050656
        observed_mean = self.network.mean_sinuosity
        self.assertEqual(observed_mean, known_mean)

        known_std = 0.07938019212245889
        observed_std = self.network.std_sinuosity
        self.assertEqual(observed_std, known_std)

    def test_sine_graph_sinuosity(self):
        known_sinuosity = [1.2105497715794307, 1.2105497715794304]
        observed_sinuosity = list(self.graph.s_data["sinuosity"])
        self.assertEqual(observed_sinuosity, known_sinuosity)

    def test_sine_graph_sinuosity_stats(self):
        known_max = 1.2105497715794307
        observed_max = self.graph.max_sinuosity
        self.assertEqual(observed_max, known_max)

        known_min = 1.2105497715794304
        observed_min = self.graph.min_sinuosity
        self.assertEqual(observed_min, known_min)

        known_mean = 1.2105497715794304
        observed_mean = self.graph.mean_sinuosity
        self.assertEqual(observed_mean, known_mean)

        known_std = 2.220446049250313e-16
        observed_std = self.graph.std_sinuosity
        self.assertEqual(observed_std, known_std)

    def test_sine_network_node_degree_stats(self):
        known_max = 2
        observed_max = self.network.max_node_degree
        self.assertEqual(observed_max, known_max)

        known_min = 1
        observed_min = self.network.min_node_degree
        self.assertEqual(observed_min, known_min)

        known_mean = 1.3333333333333333
        observed_mean = self.network.mean_node_degree
        self.assertEqual(observed_mean, known_mean)

        known_std = 0.5163977794943223
        observed_std = self.network.std_node_degree
        self.assertEqual(observed_std, known_std)

    def test_sine_graph_node_degree_stats(self):
        known_max = 1
        observed_max = self.graph.max_node_degree
        self.assertEqual(observed_max, known_max)

        known_min = 1
        observed_min = self.graph.min_node_degree
        self.assertEqual(observed_min, known_min)

        known_mean = 1.0
        observed_mean = self.graph.mean_node_degree
        self.assertEqual(observed_mean, known_mean)

        known_std = 0.0
        observed_std = self.graph.std_node_degree
        self.assertEqual(observed_std, known_std)


class TestNetworkConnectivityLattice1x1(unittest.TestCase):
    def setUp(self):
        lat1 = tigernet.generate_lattice(n_hori_lines=1, n_vert_lines=1)
        kws = {"n_hori_lines": 1, "n_vert_lines": 1, "bounds": [6, 6, 8, 8]}
        lat2 = tigernet.generate_lattice(**kws)
        self.lats = lat1.append(lat2)
        self.lats.reset_index(drop=True, inplace=True)

    def test_lattice_network_wcomps_connectivity(self):
        # with recorded components
        net_wcomps = tigernet.Network(s_data=self.lats.copy(), record_components=True)
        with self.assertWarns(UserWarning):
            net_wcomps.calc_net_stats(conn_stat="all")

        known_alpha = 0.0
        observed_alpha = net_wcomps.alpha
        self.assertEqual(observed_alpha, known_alpha)

        known_beta = 0.8
        observed_beta = net_wcomps.beta
        self.assertEqual(observed_beta, known_beta)

        known_gamma = 0.3333333333333333
        observed_gamma = net_wcomps.gamma
        self.assertEqual(observed_gamma, known_gamma)

        known_eta = 2.75
        observed_eta = net_wcomps.eta
        self.assertEqual(observed_eta, known_eta)

    def test_lattice_network_wcomps_alpha(self):
        # with recorded components
        net_wcomps = tigernet.Network(s_data=self.lats.copy(), record_components=True)
        with self.assertWarns(UserWarning):
            net_wcomps.calc_net_stats(conn_stat="alpha")

        known_alpha = 0.0
        observed_alpha = net_wcomps.alpha
        self.assertEqual(observed_alpha, known_alpha)

    def test_lattice_network_wcomps_beta(self):
        # with recorded components
        net_wcomps = tigernet.Network(s_data=self.lats.copy(), record_components=True)
        with self.assertWarns(UserWarning):
            net_wcomps.calc_net_stats(conn_stat="beta")

        known_beta = 0.8
        observed_beta = net_wcomps.beta
        self.assertEqual(observed_beta, known_beta)

    def test_lattice_network_wcomps_gamma(self):
        # with recorded components
        net_wcomps = tigernet.Network(s_data=self.lats.copy(), record_components=True)
        with self.assertWarns(UserWarning):
            net_wcomps.calc_net_stats(conn_stat="gamma")

        known_gamma = 0.3333333333333333
        observed_gamma = net_wcomps.gamma
        self.assertEqual(observed_gamma, known_gamma)

    def test_lattice_network_wcomps_eta(self):
        # with recorded components
        net_wcomps = tigernet.Network(s_data=self.lats.copy(), record_components=True)
        with self.assertWarns(UserWarning):
            net_wcomps.calc_net_stats(conn_stat="eta")

        known_eta = 2.75
        observed_eta = net_wcomps.eta
        self.assertEqual(observed_eta, known_eta)


class TestNetworkEntropyLattice1x1(unittest.TestCase):
    def setUp(self):
        self.lat = tigernet.generate_lattice(n_hori_lines=1, n_vert_lines=1)

    def test_lattice_network_segment_entropy_xvariation(self):
        net = tigernet.Network(s_data=self.lat.copy())
        net.calc_entropy("MTFCC", "s_data")

        known_entropies = {"S1400": 0.0}
        observed_entropies = net.mtfcc_entropies
        self.assertEqual(observed_entropies, known_entropies)

        known_entropy = -0.0
        observed_entropy = net.network_mtfcc_entropy
        self.assertEqual(observed_entropy, known_entropy)

    def test_lattice_network_segment_entropy_wvariation(self):
        _lat = self.lat.copy()
        _lat["MTFCC"] = ["S1100", "S1200", "S1400", "S1700"]
        net = tigernet.Network(s_data=_lat)
        net.calc_entropy("MTFCC", "s_data")

        known_entropies = {"S1100": -0.5, "S1200": -0.5, "S1400": -0.5, "S1700": -0.5}
        observed_entropies = net.mtfcc_entropies
        self.assertEqual(observed_entropies, known_entropies)

        known_entropy = 2.0
        observed_entropy = net.network_mtfcc_entropy
        self.assertEqual(observed_entropy, known_entropy)

    def test_lattice_network_node_entropy(self):
        _lat = self.lat.copy()
        net = tigernet.Network(s_data=_lat)
        net.calc_entropy("degree", "n_data")

        known_entropies = {1: -0.2575424759098898, 4: -0.46438561897747244}
        observed_entropies = net.degree_entropies
        for k, v in known_entropies.items():
            self.assertAlmostEqual(observed_entropies[k], v)

        known_entropy = 0.7219280948873623
        observed_entropy = net.network_degree_entropy
        self.assertAlmostEqual(observed_entropy, known_entropy)


class TestNetworkDistanceMetricsLattice1x1(unittest.TestCase):
    def setUp(self):
        self.lat = tigernet.generate_lattice(n_hori_lines=1, n_vert_lines=1)
        self.net = tigernet.Network(s_data=self.lat)
        self.net.cost_matrix()
        self.net.calc_net_stats()

    def test_network_radius(self):
        known_pair, known_radius = [(0, 1, 1, 1, 1, 2, 3, 4), 4.5]
        observed_pair, observed_radius = self.net.radius
        self.assertEqual(observed_pair, known_pair)
        self.assertAlmostEqual(observed_radius, known_radius)

    def test_network_diameter(self):
        known_pair, known_diameter = [(0, 0, 0, 2, 2, 2, 3, 3, 3, 4, 4, 4), 9.0]
        observed_pair, observed_diameter = self.net.diameter
        self.assertEqual(observed_pair, known_pair)
        self.assertAlmostEqual(observed_diameter, known_diameter)

    def test_network_total_network_distance(self):
        known_distance = 144.0
        observed_distance = self.net.d_net
        self.assertAlmostEqual(observed_distance, known_distance)

    def test_network_total_euclidean_distance(self):
        known_distance = 122.91168824543142
        observed_distance = self.net.d_euc
        self.assertAlmostEqual(observed_distance, known_distance, 3)

    def test_network_circuity(self):
        known_circuity = 1.17157287525381
        observed_circuity = self.net.circuity
        self.assertAlmostEqual(observed_circuity, known_circuity, 3)


##########################################################################################
# Empirical testing
##########################################################################################


class TestNetworkStatsEmpirical(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_empirical_simplified_wcm)
        self.network.calc_net_stats()

    def test_network_sinuosity(self):
        known_sinuosity = [
            1.1814786403587332,
            1.0,
            1.0002181434117707,
            1.000964242197008,
            1.1293480862834322,
        ]
        observed_sinuosity = list(self.network.s_data["sinuosity"][:5])
        for k, o in zip(known_sinuosity, observed_sinuosity):
            self.assertAlmostEqual(o, k)

    def test_network_sinuosity_stats(self):
        known_max = 4.479497558172366
        observed_max = self.network.max_sinuosity
        self.assertAlmostEqual(observed_max, known_max)

        known_min = 1.0
        observed_min = self.network.min_sinuosity
        self.assertEqual(observed_min, known_min)

        known_mean = 1.1300036198574728
        observed_mean = self.network.mean_sinuosity
        self.assertAlmostEqual(observed_mean, known_mean)

        known_std = 0.48197915355062204
        observed_std = self.network.std_sinuosity
        self.assertAlmostEqual(observed_std, known_std)

    def test_network_node_degree_stats(self):
        known_max = 5
        observed_max = self.network.max_node_degree
        self.assertEqual(observed_max, known_max)

        known_min = 1
        observed_min = self.network.min_node_degree
        self.assertEqual(observed_min, known_min)

        known_mean = 2.4125874125874125
        observed_mean = self.network.mean_node_degree
        self.assertAlmostEqual(observed_mean, known_mean)

        known_std = 1.0650997569862783
        observed_std = self.network.std_node_degree
        self.assertAlmostEqual(observed_std, known_std)


class TestNetworkConnectivityEmpirical(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_empirical_simplified)
        with self.assertWarns(UserWarning):
            self.network.calc_net_stats(conn_stat="all")

    def test_lattice_network_wcomps_connectivity(self):
        known_alpha = 0.10582010582010581
        observed_alpha = self.network.alpha
        self.assertAlmostEqual(observed_alpha, known_alpha)

        known_beta = 1.2062937062937062
        observed_beta = self.network.beta
        self.assertAlmostEqual(observed_beta, known_beta)

        known_gamma = 0.40492957746478875
        observed_gamma = self.network.gamma
        self.assertAlmostEqual(observed_gamma, known_gamma)

        known_eta = 217.00458598445144
        observed_eta = self.network.eta
        self.assertAlmostEqual(observed_eta, known_eta)


class TestNetworkEntropyEmpirical(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_empirical_lcc)
        self.network.calc_entropy("MTFCC", "s_data")
        self.network.calc_entropy("degree", "n_data")

    def test_network_entropy_variation(self):
        known_entropies = {
            "S1630": -0.15869726894257252,
            "S1100": -0.08968927494169666,
            "S1400": -0.2700938933248304,
            "S1200": -0.42847006925102654,
        }
        observed_entropies = self.network.mtfcc_entropies
        for mtfcc_type, known_entropy_val in known_entropies.items():
            self.assertAlmostEqual(observed_entropies[mtfcc_type], known_entropy_val)

        known_entropy = 0.9469505064601262
        observed_entropy = self.network.network_mtfcc_entropy
        self.assertAlmostEqual(observed_entropy, known_entropy)

    def test_network_node_entropy(self):
        known_entropies = {
            3: -0.5144720846413561,
            5: -0.024261331884622785,
            4: -0.2796123512080418,
            1: -0.5148474076720833,
            2: -0.4433974870650428,
        }
        observed_entropies = self.network.degree_entropies
        for k, v in known_entropies.items():
            self.assertAlmostEqual(observed_entropies[k], v)

        known_entropy = 1.7765906624711467
        observed_entropy = self.network.network_degree_entropy
        self.assertAlmostEqual(observed_entropy, known_entropy)


class TestNetworkDistanceMetricsEmpiricalGDF(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_empirical_simplified_wcm)
        self.network.calc_net_stats()

    def test_network_radius(self):
        known_pair, known_radius = [(130, 147), 9.943247035238207]
        observed_pair, observed_radius = self.network.radius
        self.assertEqual(observed_pair, known_pair)
        self.assertAlmostEqual(observed_radius, known_radius, 3)

    def test_network_diameter(self):
        known_pair, known_diameter = [(120, 11), 7519.207911226202]
        observed_pair, observed_diameter = self.network.diameter
        self.assertEqual(observed_pair, known_pair)
        self.assertAlmostEqual(observed_diameter, known_diameter, 3)

    def test_network_total_network_distance(self):
        known_distance = 223504355.29578546
        observed_distance = self.network.d_net
        self.assertAlmostEqual(observed_distance, known_distance, 3)

    def test_network_total_euclidean_distance(self):
        known_distance = 143326889.90408167
        observed_distance = self.network.d_euc
        self.assertAlmostEqual(observed_distance, known_distance, 3)

    def test_network_circuity(self):
        known_circuity = 1.5594028130057156
        observed_circuity = self.network.circuity
        self.assertAlmostEqual(observed_circuity, known_circuity, 3)


if __name__ == "__main__":
    unittest.main()



"""Synthetic observation data testing.
"""

import copy
import unittest
import geopandas
import numpy
from shapely.geometry import Point

import tigernet
from .network_objects import network_lattice_1x1_small


####################################################################################
############################# ORIG-XXXX--Segments ##################################
####################################################################################


class TestSyntheticObservationsOrigToXXXXSegments(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_lattice_1x1_small)

        # generate synthetic observations
        pts = [Point(1, 1), Point(3, 1), Point(1, 3), Point(3, 3)]
        obs = geopandas.GeoDataFrame({"obs_id": ["a", "b", "c", "d"]}, geometry=pts)

        # associate observations with the network
        args = self.network, obs.copy()
        kwargs = {"df_name": "obs1", "df_key": "obs_id"}
        self.net_obs = tigernet.Observations(*args, **kwargs)

    def test_net_no_snap(self):
        known_mtx = numpy.array(
            [
                [0.0, 0.0, 2.0, 2.0],
                [0.0, 0.0, 2.0, 2.0],
                [2.0, 2.0, 0.0, 0.0],
                [2.0, 2.0, 0.0, 0.0],
            ]
        )
        args = copy.deepcopy(self.net_obs), copy.deepcopy(self.network)
        kwargs = {
            "destination_observations": None,
            "snap_dist": False,
            "distance_type": "network",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 16.0
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)

    def test_net_snap(self):
        known_mtx = numpy.array(
            [
                [0.0, 2.0, 4.0, 4.0],
                [2.0, 0.0, 4.0, 4.0],
                [4.0, 4.0, 0.0, 2.0],
                [4.0, 4.0, 2.0, 0.0],
            ]
        )
        args = copy.deepcopy(self.net_obs), copy.deepcopy(self.network)
        kwargs = {
            "destination_observations": None,
            "snap_dist": True,
            "distance_type": "network",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 40.0
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)

    def test_euc_no_snap(self):
        known_mtx = numpy.array(
            [
                [0.0, 0.0, 2.0, 2.0],
                [0.0, 0.0, 2.0, 2.0],
                [2.0, 2.0, 0.0, 0.0],
                [2.0, 2.0, 0.0, 0.0],
            ]
        )
        args = copy.deepcopy(self.net_obs), copy.deepcopy(self.network)
        kwargs = {
            "destination_observations": None,
            "snap_dist": False,
            "distance_type": "euclidean",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 16.0
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)

    def test_euc_snap(self):
        known_mtx = numpy.array(
            [
                [0.0, 0.0, 2.0, 2.0],
                [0.0, 0.0, 2.0, 2.0],
                [2.0, 2.0, 0.0, 0.0],
                [2.0, 2.0, 0.0, 0.0],
            ]
        )
        args = copy.deepcopy(self.net_obs), copy.deepcopy(self.network)
        kwargs = {
            "destination_observations": None,
            "snap_dist": True,
            "distance_type": "euclidean",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 16.0
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)


####################################################################################
############################# ORIG-XXXX--Nodes #####################################
####################################################################################


class TestSyntheticObservationsOrigToXXXXNodes(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_lattice_1x1_small)

        # generate synthetic observations
        pts = [Point(1, 1), Point(3, 1), Point(1, 3), Point(3, 3)]
        obs = geopandas.GeoDataFrame({"obs_id": ["a", "b", "c", "d"]}, geometry=pts)

        # associate observations with the network
        args = self.network, obs.copy()
        kwargs = {"df_name": "obs1", "df_key": "obs_id", "snap_to": "nodes"}
        self.net_obs = tigernet.Observations(*args, **kwargs)

    def test_net_no_snap(self):
        known_mtx = numpy.array(
            [
                [0.0, 0.0, 0.0, 0.0],
                [0.0, 0.0, 0.0, 0.0],
                [0.0, 0.0, 0.0, 0.0],
                [0.0, 0.0, 0.0, 0.0],
            ]
        )
        args = self.net_obs, self.network
        kwargs = {
            "destination_observations": None,
            "snap_dist": False,
            "distance_type": "network",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 0.0
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)

    def test_net_snap(self):
        known_mtx = numpy.array(
            [
                [0.0, 2.82842712, 2.82842712, 2.82842712],
                [2.82842712, 0.0, 2.82842712, 2.82842712],
                [2.82842712, 2.82842712, 0.0, 2.82842712],
                [2.82842712, 2.82842712, 2.82842712, 0.0],
            ]
        )
        args = self.net_obs, self.network
        kwargs = {
            "destination_observations": None,
            "snap_dist": True,
            "distance_type": "network",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 33.941125496954285
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)

    def test_euc_no_snap(self):
        known_mtx = numpy.array(
            [
                [0.0, 0.0, 0.0, 0.0],
                [0.0, 0.0, 0.0, 0.0],
                [0.0, 0.0, 0.0, 0.0],
                [0.0, 0.0, 0.0, 0.0],
            ]
        )
        args = self.net_obs, self.network
        kwargs = {
            "destination_observations": None,
            "snap_dist": False,
            "distance_type": "euclidean",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 0.0
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)

    def test_euc_snap(self):
        known_mtx = numpy.array(
            [
                [0.0, 0.0, 0.0, 0.0],
                [0.0, 0.0, 0.0, 0.0],
                [0.0, 0.0, 0.0, 0.0],
                [0.0, 0.0, 0.0, 0.0],
            ]
        )
        args = self.net_obs, self.network
        kwargs = {
            "destination_observations": None,
            "snap_dist": True,
            "distance_type": "euclidean",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 0.0
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)


####################################################################################
############################# ORIG-DEST--Segments ##################################
####################################################################################


class TestSyntheticObservationsOrigToDestSegments(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_lattice_1x1_small)

        # generate synthetic origins
        obs1 = tigernet.generate_obs(5, self.network.s_data)
        obs1["obs_id"] = ["a", "b", "c", "d", "e"]
        # generate synthetic destinations
        obs2 = tigernet.generate_obs(3, self.network.s_data, seed=1)
        obs2["obs_id"] = ["z", "y", "x"]

        # associate origins with the network
        args = self.network, obs1.copy()
        kwargs = {"df_name": "obs1", "df_key": "obs_id"}
        self.net_obs1 = tigernet.Observations(*args, **kwargs)

        # associate destinations with the network
        args = self.network, obs2.copy()
        kwargs = {"df_name": "obs2", "df_key": "obs_id"}
        self.net_obs2 = tigernet.Observations(*args, **kwargs)

    def test_net_no_snap(self):
        known_mtx = numpy.array(
            [
                [0.02054051, 2.86029997, 2.49140309],
                [1.29235148, 2.41059601, 2.04169913],
                [0.29772152, 2.58311895, 2.21422207],
                [0.68579403, 3.5666345, 3.19773762],
                [2.73594902, 3.85419354, 3.48529666],
            ]
        )
        args = copy.deepcopy(self.net_obs1), copy.deepcopy(self.network)
        kwargs = {
            "destination_observations": self.net_obs2,
            "snap_dist": False,
            "distance_type": "network",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 33.737558095596384
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)

    def test_net_snap(self):
        known_mtx = numpy.array(
            [
                [0.54770651, 3.84622369, 4.09963354],
                [1.80379619, 3.38079845, 3.63420829],
                [0.93501431, 3.67916947, 3.93257931],
                [1.26735717, 4.60695537, 4.86036522],
                [3.53409492, 5.11109718, 5.36450702],
            ]
        )
        args = copy.deepcopy(self.net_obs1), copy.deepcopy(self.network)
        kwargs = {
            "destination_observations": self.net_obs2,
            "snap_dist": True,
            "distance_type": "network",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 50.60350662252801
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)

    def test_euc_no_snap(self):
        known_mtx = numpy.array(
            [
                [0.02054051, 2.17694135, 2.49140309],
                [0.97244594, 2.41059601, 1.68165696],
                [0.29772152, 2.08296224, 2.21422207],
                [0.68579403, 2.54046208, 3.19773762],
                [2.05339149, 3.85419354, 2.46956183],
            ]
        )
        args = copy.deepcopy(self.net_obs1), copy.deepcopy(self.network)
        kwargs = {
            "destination_observations": self.net_obs2,
            "snap_dist": False,
            "distance_type": "euclidean",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 29.14963026553491
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)

    def test_euc_snap(self):
        known_mtx = numpy.array(
            [
                [0.02054051, 2.17694135, 2.49140309],
                [0.97244594, 2.41059601, 1.68165696],
                [0.29772152, 2.08296224, 2.21422207],
                [0.68579403, 2.54046208, 3.19773762],
                [2.05339149, 3.85419354, 2.46956183],
            ]
        )
        args = copy.deepcopy(self.net_obs1), copy.deepcopy(self.network)
        kwargs = {
            "destination_observations": self.net_obs2,
            "snap_dist": True,
            "distance_type": "euclidean",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 29.14963026553491
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)


####################################################################################
############################# ORIG-DEST--Nodes #####################################
####################################################################################


class TestSyntheticObservationsOrigToDestNodes(unittest.TestCase):
    def setUp(self):
        self.network = copy.deepcopy(network_lattice_1x1_small)

        # generate synthetic origins
        obs1 = tigernet.generate_obs(5, self.network.s_data)
        obs1["obs_id"] = ["a", "b", "c", "d", "e"]
        # generate synthetic destinations
        obs2 = tigernet.generate_obs(3, self.network.s_data, seed=1)
        obs2["obs_id"] = ["z", "y", "x"]

        # associate origins with the network
        args = self.network, obs1.copy()
        kwargs = {"df_name": "obs1", "df_key": "obs_id", "snap_to": "nodes"}
        self.net_obs1 = tigernet.Observations(*args, **kwargs)

        # associate destinations with the network
        args = self.network, obs2.copy()
        kwargs = {"df_name": "obs2", "df_key": "obs_id", "snap_to": "nodes"}
        self.net_obs2 = tigernet.Observations(*args, **kwargs)

    def test_net_no_snap(self):
        known_mtx = numpy.array(
            [
                [0.0, 2.0, 2.0],
                [0.0, 2.0, 2.0],
                [0.0, 2.0, 2.0],
                [2.0, 4.0, 4.0],
                [2.0, 4.0, 4.0],
            ]
        )
        args = self.net_obs1, self.network
        kwargs = {
            "destination_observations": self.net_obs2,
            "snap_dist": False,
            "distance_type": "network",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 32.0
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)

    def test_net_snap(self):
        known_mtx = numpy.array(
            [
                [1.8243534, 3.67329521, 4.34307909],
                [1.3902779, 3.23921971, 3.90900359],
                [1.60037734, 3.44931915, 4.11910303],
                [3.44146299, 5.2904048, 5.96018868],
                [3.43009305, 5.27903486, 5.94881874],
            ]
        )
        args = self.net_obs1, self.network
        kwargs = {
            "destination_observations": self.net_obs2,
            "snap_dist": True,
            "distance_type": "network",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 56.89803154575887
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)

    def test_euc_no_snap(self):
        known_mtx = numpy.array(
            [
                [0.0, 2.0, 2.0],
                [0.0, 2.0, 2.0],
                [0.0, 2.0, 2.0],
                [2.0, 2.82842712, 4.0],
                [2.0, 4.0, 2.82842712],
            ]
        )
        args = self.net_obs1, self.network
        kwargs = {
            "destination_observations": self.net_obs2,
            "snap_dist": False,
            "distance_type": "euclidean",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 29.656854249492383
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)

    def test_euc_snap(self):
        known_mtx = numpy.array(
            [
                [0.0, 2.0, 2.0],
                [0.0, 2.0, 2.0],
                [0.0, 2.0, 2.0],
                [2.0, 2.82842712, 4.0],
                [2.0, 4.0, 2.82842712],
            ]
        )
        args = self.net_obs1, self.network
        kwargs = {
            "destination_observations": self.net_obs2,
            "snap_dist": True,
            "distance_type": "euclidean",
        }
        observed_mtx = tigernet.obs2obs_cost_matrix(*args, **kwargs)
        numpy.testing.assert_array_almost_equal(observed_mtx, known_mtx)

        known_mtx_sum = 29.656854249492383
        observed_mtx_sum = observed_mtx.sum()
        self.assertAlmostEqual(observed_mtx_sum, known_mtx_sum)


if __name__ == "__main__":
    unittest.main()



"""Cost matrix and shorted path tree testing.
"""


import copy
import unittest
import numpy

from .network_objects import network_lattice_1x1_wcm_attr
from .network_objects import network_lattice_1x1_wpaths_attr
from .network_objects import network_lattice_1x1_wcm_var
from .network_objects import network_lattice_1x1_wpaths_var

from .network_objects import network_lattice_2x1x1_wcm_attr
from .network_objects import network_lattice_2x1x1_wpaths_attr
from .network_objects import network_lattice_2x1x1_wcm_var
from .network_objects import network_lattice_2x1x1_wpaths_var

from .network_objects import graph_barb_wcm_copy_attr
from .network_objects import graph_barb_wpaths_copy_attr
from .network_objects import graph_barb_wcm_copy_var
from .network_objects import graph_barb_wpaths_copy_var

from .network_objects import network_barb_wcm_inplace_attr
from .network_objects import network_barb_wpaths_inplace_attr
from .network_objects import network_barb_wcm_inplace_var
from .network_objects import network_barb_wpaths_inplace_var

from .network_objects import network_empirical_simplified_wcm


inf = numpy.inf


class TestNetworkCostMatrixLattice1x1_1(unittest.TestCase):
    def setUp(self):
        # generate cost matrix without paths as an attribute
        self.network_mtx = copy.deepcopy(network_lattice_1x1_wcm_attr)

        # generate cost matrix with paths as an attribute
        self.network_mtx_paths = copy.deepcopy(network_lattice_1x1_wpaths_attr)

        # generate cost matrix without paths as an attribute
        self.matrix = copy.deepcopy(network_lattice_1x1_wcm_var)

        # generate cost matrix with paths as an attribute
        self.paths = copy.deepcopy(network_lattice_1x1_wpaths_var)

    def test_network_cost_matrix_attr(self):
        known_cost_matrix = numpy.array(
            [
                [0.0, 4.5, 9.0, 9.0, 9.0],
                [4.5, 0.0, 4.5, 4.5, 4.5],
                [9.0, 4.5, 0.0, 9.0, 9.0],
                [9.0, 4.5, 9.0, 0.0, 9.0],
                [9.0, 4.5, 9.0, 9.0, 0.0],
            ]
        )
        observed_cost_matrix = self.network_mtx.n2n_matrix
        numpy.testing.assert_array_equal(observed_cost_matrix, known_cost_matrix)

    def test_network_paths_attr(self):
        known_paths = {
            0: {0: [0], 1: [0], 2: [1, 0], 3: [1, 0], 4: [1, 0]},
            1: {0: [1], 1: [1], 2: [1], 3: [1], 4: [1]},
            2: {0: [1, 2], 1: [2], 2: [2], 3: [1, 2], 4: [1, 2]},
            3: {0: [1, 3], 1: [3], 2: [1, 3], 3: [3], 4: [1, 3]},
            4: {0: [1, 4], 1: [4], 2: [1, 4], 3: [1, 4], 4: [4]},
        }
        observed_paths = self.network_mtx_paths.n2n_paths
        self.assertEqual(observed_paths, known_paths)

    def test_network_cost_matrix_return(self):
        known_cost_matrix = numpy.array(
            [
                [0.0, 4.5, 9.0, 9.0, 9.0],
                [4.5, 0.0, 4.5, 4.5, 4.5],
                [9.0, 4.5, 0.0, 9.0, 9.0],
                [9.0, 4.5, 9.0, 0.0, 9.0],
                [9.0, 4.5, 9.0, 9.0, 0.0],
            ]
        )
        observed_cost_matrix = self.matrix
        numpy.testing.assert_array_equal(observed_cost_matrix, known_cost_matrix)

    def test_network_paths_return(self):
        known_paths = {
            0: {0: [0], 1: [0], 2: [1, 0], 3: [1, 0], 4: [1, 0]},
            1: {0: [1], 1: [1], 2: [1], 3: [1], 4: [1]},
            2: {0: [1, 2], 1: [2], 2: [2], 3: [1, 2], 4: [1, 2]},
            3: {0: [1, 3], 1: [3], 2: [1, 3], 3: [3], 4: [1, 3]},
            4: {0: [1, 4], 1: [4], 2: [1, 4], 3: [1, 4], 4: [4]},
        }
        observed_paths = self.paths
        self.assertEqual(observed_paths, known_paths)


class TestNetworkCostMatrixLattice1x1_2(unittest.TestCase):
    def setUp(self):
        # generate cost matrix without paths as an attribute
        self.network_mtx = copy.deepcopy(network_lattice_2x1x1_wcm_attr)

        # generate cost matrix with paths as an attribute
        self.network_mtx_paths = copy.deepcopy(network_lattice_2x1x1_wpaths_attr)

        # generate cost matrix without paths as an attribute
        self.matrix = copy.deepcopy(network_lattice_2x1x1_wcm_var)

        # generate cost matrix with paths as an attribute
        self.paths = copy.deepcopy(network_lattice_2x1x1_wpaths_var)

    def test_network_cost_matrix_attr(self):
        known_cost_matrix = numpy.array(
            [
                [0.0, 4.5, 9.0, 9.0, 9.0, inf, inf, inf, inf, inf],
                [4.5, 0.0, 4.5, 4.5, 4.5, inf, inf, inf, inf, inf],
                [9.0, 4.5, 0.0, 9.0, 9.0, inf, inf, inf, inf, inf],
                [9.0, 4.5, 9.0, 0.0, 9.0, inf, inf, inf, inf, inf],
                [9.0, 4.5, 9.0, 9.0, 0.0, inf, inf, inf, inf, inf],
                [inf, inf, inf, inf, inf, 0.0, 1.0, 2.0, 2.0, 2.0],
                [inf, inf, inf, inf, inf, 1.0, 0.0, 1.0, 1.0, 1.0],
                [inf, inf, inf, inf, inf, 2.0, 1.0, 0.0, 2.0, 2.0],
                [inf, inf, inf, inf, inf, 2.0, 1.0, 2.0, 0.0, 2.0],
                [inf, inf, inf, inf, inf, 2.0, 1.0, 2.0, 2.0, 0.0],
            ]
        )
        observed_cost_matrix = self.network_mtx.n2n_matrix
        numpy.testing.assert_array_equal(observed_cost_matrix, known_cost_matrix)

    def test_network_paths_attr(self):
        known_paths = {
            0: {
                0: [0],
                1: [0],
                2: [1, 0],
                3: [1, 0],
                4: [1, 0],
                5: [5],
                6: [6],
                7: [7],
                8: [8],
                9: [9],
            },
            1: {
                0: [1],
                1: [1],
                2: [1],
                3: [1],
                4: [1],
                5: [5],
                6: [6],
                7: [7],
                8: [8],
                9: [9],
            },
            2: {
                0: [1, 2],
                1: [2],
                2: [2],
                3: [1, 2],
                4: [1, 2],
                5: [5],
                6: [6],
                7: [7],
                8: [8],
                9: [9],
            },
            3: {
                0: [1, 3],
                1: [3],
                2: [1, 3],
                3: [3],
                4: [1, 3],
                5: [5],
                6: [6],
                7: [7],
                8: [8],
                9: [9],
            },
            4: {
                0: [1, 4],
                1: [4],
                2: [1, 4],
                3: [1, 4],
                4: [4],
                5: [5],
                6: [6],
                7: [7],
                8: [8],
                9: [9],
            },
            5: {
                0: [0],
                1: [1],
                2: [2],
                3: [3],
                4: [4],
                5: [5],
                6: [5],
                7: [6, 5],
                8: [6, 5],
                9: [6, 5],
            },
            6: {
                0: [0],
                1: [1],
                2: [2],
                3: [3],
                4: [4],
                5: [6],
                6: [6],
                7: [6],
                8: [6],
                9: [6],
            },
            7: {
                0: [0],
                1: [1],
                2: [2],
                3: [3],
                4: [4],
                5: [6, 7],
                6: [7],
                7: [7],
                8: [6, 7],
                9: [6, 7],
            },
            8: {
                0: [0],
                1: [1],
                2: [2],
                3: [3],
                4: [4],
                5: [6, 8],
                6: [8],
                7: [6, 8],
                8: [8],
                9: [6, 8],
            },
            9: {
                0: [0],
                1: [1],
                2: [2],
                3: [3],
                4: [4],
                5: [6, 9],
                6: [9],
                7: [6, 9],
                8: [6, 9],
                9: [9],
            },
        }
        observed_paths = self.network_mtx_paths.n2n_paths
        self.assertEqual(observed_paths, known_paths)

    def test_network_cost_matrix_return(self):
        known_cost_matrix = numpy.array(
            [
                [0.0, 4.5, 9.0, 9.0, 9.0, inf, inf, inf, inf, inf],
                [4.5, 0.0, 4.5, 4.5, 4.5, inf, inf, inf, inf, inf],
                [9.0, 4.5, 0.0, 9.0, 9.0, inf, inf, inf, inf, inf],
                [9.0, 4.5, 9.0, 0.0, 9.0, inf, inf, inf, inf, inf],
                [9.0, 4.5, 9.0, 9.0, 0.0, inf, inf, inf, inf, inf],
                [inf, inf, inf, inf, inf, 0.0, 1.0, 2.0, 2.0, 2.0],
                [inf, inf, inf, inf, inf, 1.0, 0.0, 1.0, 1.0, 1.0],
                [inf, inf, inf, inf, inf, 2.0, 1.0, 0.0, 2.0, 2.0],
                [inf, inf, inf, inf, inf, 2.0, 1.0, 2.0, 0.0, 2.0],
                [inf, inf, inf, inf, inf, 2.0, 1.0, 2.0, 2.0, 0.0],
            ]
        )
        observed_cost_matrix = self.matrix
        numpy.testing.assert_array_equal(observed_cost_matrix, known_cost_matrix)

    def test_network_paths_return(self):
        known_paths = {
            0: {
                0: [0],
                1: [0],
                2: [1, 0],
                3: [1, 0],
                4: [1, 0],
                5: [5],
                6: [6],
                7: [7],
                8: [8],
                9: [9],
            },
            1: {
                0: [1],
                1: [1],
                2: [1],
                3: [1],
                4: [1],
                5: [5],
                6: [6],
                7: [7],
                8: [8],
                9: [9],
            },
            2: {
                0: [1, 2],
                1: [2],
                2: [2],
                3: [1, 2],
                4: [1, 2],
                5: [5],
                6: [6],
                7: [7],
                8: [8],
                9: [9],
            },
            3: {
                0: [1, 3],
                1: [3],
                2: [1, 3],
                3: [3],
                4: [1, 3],
                5: [5],
                6: [6],
                7: [7],
                8: [8],
                9: [9],
            },
            4: {
                0: [1, 4],
                1: [4],
                2: [1, 4],
                3: [1, 4],
                4: [4],
                5: [5],
                6: [6],
                7: [7],
                8: [8],
                9: [9],
            },
            5: {
                0: [0],
                1: [1],
                2: [2],
                3: [3],
                4: [4],
                5: [5],
                6: [5],
                7: [6, 5],
                8: [6, 5],
                9: [6, 5],
            },
            6: {
                0: [0],
                1: [1],
                2: [2],
                3: [3],
                4: [4],
                5: [6],
                6: [6],
                7: [6],
                8: [6],
                9: [6],
            },
            7: {
                0: [0],
                1: [1],
                2: [2],
                3: [3],
                4: [4],
                5: [6, 7],
                6: [7],
                7: [7],
                8: [6, 7],
                9: [6, 7],
            },
            8: {
                0: [0],
                1: [1],
                2: [2],
                3: [3],
                4: [4],
                5: [6, 8],
                6: [8],
                7: [6, 8],
                8: [8],
                9: [6, 8],
            },
            9: {
                0: [0],
                1: [1],
                2: [2],
                3: [3],
                4: [4],
                5: [6, 9],
                6: [9],
                7: [6, 9],
                8: [6, 9],
                9: [9],
            },
        }
        observed_paths = self.paths
        self.assertEqual(observed_paths, known_paths)


class TestNetworkCostMatrixSimplifyBarb(unittest.TestCase):
    def setUp(self):
        # ---------------------------------------------------- copy
        self.graph_mtx_copy = graph_barb_wcm_copy_attr
        self.graph_mtx_paths_copy = graph_barb_wpaths_copy_attr
        self.graph_matrix_copy = graph_barb_wcm_copy_var
        self.graph_paths_copy = graph_barb_wpaths_copy_var

        # ---------------------------------------------------- inplace
        self.network_mtx_inplace = network_barb_wcm_inplace_attr
        self.network_mtx_paths_inplace = network_barb_wpaths_inplace_attr
        self.network_matrix_inplace = network_barb_wcm_inplace_var
        self.network_paths_inplace = network_barb_wpaths_inplace_var

    def test_graph_copy_cost_matrix_attr(self):
        known_cost_matrix = numpy.array(
            [[0.0, 4.5, 9.0], [4.5, 0.0, 4.5], [9.0, 4.5, 0.0]]
        )
        observed_cost_matrix = self.graph_mtx_copy.n2n_matrix
        numpy.testing.assert_array_equal(observed_cost_matrix, known_cost_matrix)

    def test_graph_copy_paths_attr(self):
        known_paths = {
            0: {0: [0], 1: [0], 2: [1, 0]},
            1: {0: [1], 1: [1], 2: [1]},
            2: {0: [1, 2], 1: [2], 2: [2]},
        }
        observed_paths = self.graph_mtx_paths_copy.n2n_paths
        self.assertEqual(observed_paths, known_paths)

    def test_graph_copy_cost_matrix_return(self):
        known_cost_matrix = numpy.array(
            [[0.0, 4.5, 9.0], [4.5, 0.0, 4.5], [9.0, 4.5, 0.0]]
        )
        observed_cost_matrix = self.graph_matrix_copy
        numpy.testing.assert_array_equal(observed_cost_matrix, known_cost_matrix)

    def test_graph_copy_paths_return(self):
        known_paths = {
            0: {0: [0], 1: [0], 2: [1, 0]},
            1: {0: [1], 1: [1], 2: [1]},
            2: {0: [1, 2], 1: [2], 2: [2]},
        }
        observed_paths = self.graph_paths_copy
        self.assertEqual(observed_paths, known_paths)

    def test_graph_inplace_cost_matrix_attr(self):
        known_cost_matrix = numpy.array(
            [[0.0, 4.5, 9.0], [4.5, 0.0, 4.5], [9.0, 4.5, 0.0]]
        )
        observed_cost_matrix = self.network_mtx_inplace.n2n_matrix
        numpy.testing.assert_array_equal(observed_cost_matrix, known_cost_matrix)

    def test_graph_inplace_paths_attr(self):
        known_paths = {
            0: {0: [0], 1: [0], 2: [1, 0]},
            1: {0: [1], 1: [1], 2: [1]},
            2: {0: [1, 2], 1: [2], 2: [2]},
        }
        observed_paths = self.network_mtx_paths_inplace.n2n_paths
        self.assertEqual(observed_paths, known_paths)

    def test_graph_inplace_cost_matrix_return(self):
        known_cost_matrix = numpy.array(
            [[0.0, 4.5, 9.0], [4.5, 0.0, 4.5], [9.0, 4.5, 0.0]]
        )
        observed_cost_matrix = self.network_matrix_inplace
        numpy.testing.assert_array_equal(observed_cost_matrix, known_cost_matrix)

    def test_graph_inplace_paths_return(self):
        known_paths = {
            0: {0: [0], 1: [0], 2: [1, 0]},
            1: {0: [1], 1: [1], 2: [1]},
            2: {0: [1, 2], 1: [2], 2: [2]},
        }
        observed_paths = self.network_paths_inplace
        self.assertEqual(observed_paths, known_paths)


class TestNetworkCostMatrixEmpircalGDF(unittest.TestCase):
    def setUp(self):
        # cost matrix
        self.matrix = network_empirical_simplified_wcm.n2n_matrix

        # shortest path trees
        self.paths = network_empirical_simplified_wcm.n2n_paths

    def test_network_cost_matrix(self):
        known_cost_matrix = numpy.array(
            [
                [0.0, 212.3835341, 167.23911279, 343.31388701],
                [212.3835341, 0.0, 120.53015797, 296.60493218],
                [167.23911279, 120.53015797, 0.0, 176.07477422],
                [343.31388701, 296.60493218, 176.07477422, 0.0],
            ]
        )
        observed_cost_matrix = self.matrix[:4, :4]
        numpy.testing.assert_array_almost_equal(observed_cost_matrix, known_cost_matrix)

    def test_network_paths_1(self):
        known_src, known_dest = 0, 0
        known_path = [0]
        observed_path = self.paths[known_src][known_dest]
        self.assertEqual(observed_path, known_path)

    def test_network_paths_2(self):
        known_src, known_dest = 19, 99
        known_path = [98, 18, 19]
        observed_path = self.paths[known_src][known_dest]
        self.assertEqual(observed_path, known_path)

    def test_network_paths_3(self):
        known_src, known_dest = 20, 17
        known_path = [
            16,
            15,
            2,
            1,
            57,
            56,
            73,
            188,
            101,
            199,
            216,
            148,
            154,
            110,
            21,
            20,
        ]
        observed_path = self.paths[known_src][known_dest]
        self.assertEqual(observed_path, known_path)


if __name__ == "__main__":
    unittest.main()
